/* 29/01/12	version ver 2						*/
/* Changes:								*/
/*	Hack ad1940 code base						*/
/* 									*/
/* known Issues: All of them						*/
/* use of arrays is massively code inefficient - fix needed             */



#include "delay.h"
#include "dig_cross.h"
#include "SigmaStudioFW.h"
#include <math.h>
#include "xc.h"
#include <plib.h>
#include <p32mx450f256h.h>



/* Make this global to this function to save RAM */
int Data_Counter = 0;
long int Fixed_Point;
static unsigned char	Read_Data_Array[6];						/* Array used to hold data read during debug process */




/* Initialise ADCs  */
/* not using SPI control of these - so simple hold in reset till the clocks are stable */
/* the CS4398 in particular is pretty fussy about this */
void Init_ADC(void)
{
SetDACReset;
DelayMs(DAC_Reset_Time);		/* Hold ADC and DAC in reset for a defined period of time */
ClearDACReset;
}





/* filter type, filter Rate, freq, Q, GainGen, Gain, A0, A1, A2, B0, B1, B2*/
void Calc_DSP_Parms(int FilterType, int FilterRate, int FilterFreq, double FilterQ, int GainGen, int Gain, int *A0, int *A1, int *A2, int *B0, int *B1, int *B2)
{
    double W0, Alpha;
    double gainLinear;
    double A_gain;

    gainLinear = pow(10,(double)GainGen/100);  /* remember that filter gain is in tenths od dB*/
    W0 = 2*Pi* (double)FilterFreq/ (double)Sample_Rate;
    Alpha = sin(W0)/(2*(double)FilterQ);      /* alpha = sin(?0)/(2*Q)*/
    A_gain = pow(10, (double)Gain/40);


    if ((FilterType == LowPass) && (FilterRate == TwelvedB))
    {
     /*     a0 = 1 + alpha
            a1 = -2*cos(?0)
            a2 = 1 - alpha
            b0 = (1 - cos(?0)) * gainLinear / 2
            b1 = (1 - cos(?0)) * gainLinear
            b2 = (1 - cos(?0)) * gainLinear / 2*/
        *A0 = (int)(IIR_Scaling);
        *A1 = (int) IIR_Scaling*((2 * cos(W0))/(1 + Alpha)); /* change sign */
        *A2 = (int) IIR_Scaling*(-(1 - Alpha)/(1 + Alpha));  /* change sign */
        *B0 = (int) IIR_Scaling*(((1 - cos(W0)) * gainLinear / 2)/(1 + Alpha));
        *B1 = (int) IIR_Scaling*(((1 - cos(W0)) * gainLinear)/(1 + Alpha));
        *B2 = (int) IIR_Scaling*(((1 - cos(W0)) * gainLinear / 2)/(1 + Alpha));
    }
    else if ((FilterType == LowPass) && (FilterRate == SixdB))
    {
    /*      Coefficients
            a1 = 2.7^-?0
            b0 = gainLinear * (1.0 - a1)
            b1 = 0  */
        *A0 = (int)(IIR_Scaling);
        *A1 = (int) IIR_Scaling*pow((double)2.7, -W0);
        *A2 = (int) 0;
        *B0 = (int) IIR_Scaling*(gainLinear * (1 - pow((double)2.7, -W0)));
        *B1 = (int) 0;
        *B2 = (int) 0;
    }
    else if ((FilterType == HighPass)  && (FilterRate == TwelvedB))
    {
        /* A0 = 1 + alpha
           A1 = -2*cos(?0)
           A2 = 1 - alpha
           B0 = (1 + cos(?0)) * gainLinear / 2
           B1 = -(1 + cos(?0)) * gainLinear
           B2 = (1 + cos(?0)) * gainLinear / 2
        */
        *A0 = (int) (IIR_Scaling);
        *A1 = (int) (IIR_Scaling*((2 * cos(W0))/(1 + Alpha))); /* change sign */
        *A2 = (int) (IIR_Scaling*(-(1 - Alpha)/(1 + Alpha)));  /* change sign */
        *B0 = (int) (IIR_Scaling*(((1 + cos(W0)) * gainLinear / 2)/(1 + Alpha)));
        *B1 = (int) (IIR_Scaling*((-(1 + cos(W0)) * gainLinear)/(1 + Alpha)));
        *B2 = (int) (IIR_Scaling*(((1 + cos(W0)) * gainLinear / 2)/(1 + Alpha)));
    }
    else if ((FilterType == HighPass) && (FilterRate == SixdB))
    {
    /*
     Coefficients   a1 = 2.7^-?0
                    b0 = gainLinear * a1
                    b1 = -a1 * gainLinear
             */
        *A0 = (int)(IIR_Scaling);
        *A1 = (int) IIR_Scaling * pow((double)2.7,-W0);
        *A2 = (int) 0;
        *B0 = (int) (gainLinear * (*A1));     /* remember A1 is already scales by IIR_Scaling....*/
        *B1 = (int) (-gainLinear * (*A1));    /* remember A1 is already scales by IIR_Scaling....*/
        *B2 = (int) 0;
    }
    else if (FilterType == Peaking)
    {
        /*  A = 10^(Gain/40)
            a0 = 1 + alpha/A
            a1 = -2 * cos(?0)
            a2 = 1 - alpha/A
            b0 = (1 + alpha*A) * gainLinear
            b1 = -(2 * cos(?0)) * gainLinear
            b2 = (1 - alpha*A) * gainLinear
         */
        *A0 = (int)(IIR_Scaling);
        *A1 = (int) IIR_Scaling*((2 * cos(W0))/(1 + Alpha/A_gain)); /* change sign */
        *A2 = (int) IIR_Scaling*(((Alpha/A_gain) -1)/(1 + Alpha/A_gain));  /* change sign */
        *B0 = (int) IIR_Scaling*(((1 + Alpha*A_gain) * gainLinear)/(1 + Alpha/A_gain));
        *B1 = (int) IIR_Scaling*((-2*cos(W0) * gainLinear)/(1 + Alpha/A_gain));
        *B2 = (int) IIR_Scaling*(((1 - Alpha*A_gain) * gainLinear)/(1 + Alpha/A_gain));
    }
    else /* filtertype = none*/
    {
        *A0 = 0;
        *A1 = 0;
        *A2 = 0;
        *B0 = (int) IIR_Scaling * gainLinear;
        *B1 = 0;
        *B2 = 0;
    };
}
