/* PIC32 DDS                                                                */
/*                                                                          */
/* Go back and check if you want Interpolated output to be long long or long */
/*                                                                          */
/* V1.1 Set sample rate to 46KHz                                             */
/* Numerous tweaks to the HMI                                               */
/* A/D and D/A working as passthrough for SPI2                              s*/
/* V1.2 IIR Filter calculation and update function - in AD1940.h working     */
/*****************************************************************************/
/*                                                                           */
/* V1.3 - HMI working, Filters doing something- filter code interface to HMI */
/*        needs some serious fixing!                                         */
/*          - Volume control needs to be implemented                         */
/*          - Save function needs to be debugged - hangs                     */
/*          - Delay code needs to be implemented                             */
/*          - Logic in HMI is not too obvious - review                       */
/*          - You need to output data on SPI2!                               */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/* V1.4 - HMI working, Filters doing something-                              */
/*          - Logic in HMI is not too obvious - review                       */
/*          - You need to output data on SPI2!                               */
/*                                                                           */
/*   --> Work out optimisation of long long                                  */
/*    --> Work out why the delay to enter the ISR is inconsistent            */
/*                                                                           */
/*****************************************************************************/

#include <time.h>
#include "xc.h"
#include <plib.h>
#include "delay.h"
#include "dig_cross.h"
#include <math.h>
#include "AD1940.h"
#include <p32mx450f256h.h>
#include "stdio.h"
#include "EEPROM.h"
#include <peripheral/int.h>
#include <peripheral/ports.h>
#include "Sine_Lookup.h"


// Configuration Bit settings
// SYSCLK = 100 MHz (8MHz Crystal/ FPLLIDIV * FPLLMUL / FPLLODIV)
// PBCLK = 10 MHz
// Primary Osc w/PLL (XT+,HS+,EC+PLL)
// WDT OFF
// Other options are don't care
//
#pragma config FPLLMUL = MUL_24, FPLLIDIV = DIV_2, FPLLODIV = DIV_1, FWDTEN = OFF
#pragma config POSCMOD = HS, FNOSC = PRIPLL, FPBDIV = DIV_1
#pragma config UPLLIDIV = DIV_2, UPLLEN = ON
#pragma config JTAGEN = OFF
#pragma config FSRSSEL =  PRIORITY_3


static unsigned char Old_Rot_Status = 0;  			/* Records state of rotary control */
int Fast_Loop_Counter = Fast_Counter_Init; 	/* Used to set Fast_Cnt */
int Slow_Loop_Counter = Slow_Counter_Init; 	/* Used to see if user turning control continuously */
static unsigned char Keypress_Read = 0;			/* this is the data on key / rotation changes */
static unsigned char Key_Just_Pressed = 0;  		/* use this to keep track of which keys have been pressed */
int Fast_Change = 1;			/* use this to indicate fast change of values */


long long Acc_Reg;      /* input current sample*/
int *Out_Data;          /* pointer to output data */

data_val_struct Data_Values;                            /* This is where all the data is stored... */
char loop, test_temp;

int New_Filter_Vals;    /* use to flag new filter parms */
int Input_Mute;         /* used to mute input */
int New_a0 = 0, New_a1 = 0, New_a2 = 0, New_b0 = 0, New_b1 = 0, New_b2 = 0;       /* Biquad filter coefficients to update */

int F0_a0 = 0, F0_a1 = 0, F0_a2 = 0, F0_b0 = 0, F0_b1 = 0, F0_b2 = 0;             /* Biquad filter coefficients 0 */
int F1_a0 = 0, F1_a1 = 0, F1_a2 = 0, F1_b0 = 0, F1_b1 = 0, F1_b2 = 0;             /* Biquad filter coefficients 1 */
int F2_a0 = 0, F2_a1 = 0, F2_a2 = 0, F2_b0 = 0, F2_b1 = 0, F2_b2 = 0;             /* Biquad filter coefficients 2 */
int F3_a0 = 0, F3_a1 = 0, F3_a2 = 0, F3_b0 = 0, F3_b1 = 0, F3_b2 = 0;             /* Biquad filter coefficients 3 */
int F4_a0 = 0, F4_a1 = 0, F4_a2 = 0, F4_b0 = 0, F4_b1 = 0, F4_b2 = 0;             /* Biquad filter coefficients 4 */
int F5_a0 = 0, F5_a1 = 0, F5_a2 = 0, F5_b0 = 0, F5_b1 = 0, F5_b2 = 0;             /* Biquad filter coefficients 5 */
int F6_a0 = 0, F6_a1 = 0, F6_a2 = 0, F6_b0 = 0, F6_b1 = 0, F6_b2 = 0;             /* Biquad filter coefficients 6 */
int F7_a0 = 0, F7_a1 = 0, F7_a2 = 0, F7_b0 = 0, F7_b1 = 0, F7_b2 = 0;             /* Biquad filter coefficients 7 */

int Xn_L, Xn_R;         /* input current sample*/
int Xn1_L, Xn1_R;       /* Previous input current sample*/
int Xn2_L, Xn2_R;       /* second prvious input current sample*/
int Yn_L, Yn_R;         /* Output current sample*/
int Yn1_L, Yn1_R;       /* Previous output current sample*/
int Yn2_L, Yn2_R;       /* second previous output current sample*/

int X1n_L, X1n_R;       /* input current sample*/
int X1n1_L, X1n1_R;     /* Previous input current sample*/
int X1n2_L, X1n2_R;     /* second prvious input current sample*/
int Y1n_L, Y1n_R;       /* Output current sample*/
int Y1n1_L, Y1n1_R;     /* Previous output current sample*/
int Y1n2_L, Y1n2_R;     /* second previous output current sample*/

int X2n_L, X2n_R;       /* input current sample*/
int X2n1_L, X2n1_R;     /* Previous input current sample*/
int X2n2_L, X2n2_R;     /* second prvious input current sample*/
int Y2n_L, Y2n_R;       /* Output current sample*/
int Y2n1_L, Y2n1_R;     /* Previous output current sample*/
int Y2n2_L, Y2n2_R;     /* second previous output current sample*/

int X3n_L, X3n_R;       /* input current sample*/
int X3n1_L, X3n1_R;     /* Previous input current sample*/
int X3n2_L, X3n2_R;     /* second prvious input current sample*/
int Y3n_L, Y3n_R;       /* Output current sample*/
int Y3n1_L, Y3n1_R;     /* Previous output current sample*/
int Y3n2_L, Y3n2_R;     /* second previous output current sample*/

int X4n_L, X4n_R;       /* input current sample*/
int X4n1_L, X4n1_R;     /* Previous input current sample*/
int X4n2_L, X4n2_R;     /* second prvious input current sample*/
int Y4n_L, Y4n_R;       /* Output current sample*/
int Y4n1_L, Y4n1_R;     /* Previous output current sample*/
int Y4n2_L, Y4n2_R;     /* second previous output current sample*/

int X5n_L, X5n_R;       /* input current sample*/
int X5n1_L, X5n1_R;     /* Previous input current sample*/
int X5n2_L, X5n2_R;     /* second prvious input current sample*/
int Y5n_L, Y5n_R;       /* Output current sample*/
int Y5n1_L, Y5n1_R;     /* Previous output current sample*/
int Y5n2_L, Y5n2_R;     /* second previous output current sample*/

int X6n_L, X6n_R;       /* input current sample*/
int X6n1_L, X6n1_R;     /* Previous input current sample*/
int X6n2_L, X6n2_R;     /* second prvious input current sample*/
int Y6n_L, Y6n_R;       /* Output current sample*/
int Y6n1_L, Y6n1_R;     /* Previous output current sample*/
int Y6n2_L, Y6n2_R;     /* second previous output current sample*/

int X7n_L, X7n_R;       /* input current sample*/
int X7n1_L, X7n1_R;     /* Previous input current sample*/
int X7n2_L, X7n2_R;     /* second prvious input current sample*/
int Y7n_L, Y7n_R;       /* Output current sample*/
int Y7n1_L, Y7n1_R;     /* Previous output current sample*/
int Y7n2_L, Y7n2_R;     /* second previous output current sample*/

int CH1_Output_Data_L, CH1_Output_Data_R; /* use this for output of scale limited values*/
int CH2_Output_Data_L, CH2_Output_Data_R; /* use this for output of scale limited values*/
int CH1_Invert, CH2_Invert; /* exactly that*/
unsigned int CH1_Delay_Offset, CH2_Delay_Offset; /* Offset from input write pointer to implement delay*/
unsigned int Write_Pointer_Offset, Ch1_Rd_Offset, Ch2_Rd_Offset; /* location at which input data is written into a cyclical buffer */
int Delay_Buffer_Ch1_L[Delay_Buf_Size];  /* cyclic buffer to implement delay for L channel*/
int Delay_Buffer_Ch1_R[Delay_Buf_Size];  /* cyclic buffer to implement delay for R channel*/
int Delay_Buffer_Ch2_L[Delay_Buf_Size];  /* cyclic buffer to implement delay for L channel*/
int Delay_Buffer_Ch2_R[Delay_Buf_Size];  /* cyclic buffer to implement delay for R channel*/
int Out_Test_Temp;                       /* used for output clipping to 24 bits */

/* You might want to move these into the display LCD function - though it is really not a gut buster for these two */
char line1[] =   Line1_Init;
char line2[] =   Line2_Init;
char *line1_ptr, *line2_ptr;				/* use these to point into the arrays */

/* These are ROM constants used in bnuilding the display */
const char Blank_Char = Blank_Char_Init;            /* ROM */
const char Sel_Str[] = Line2_Select_Init;           /* ROM */
unsigned int display_temp;
const char Ch1_Str[] =  Line1_Channel_1_Init;       /* ROM */
const char Ch2_Str[] =  Line1_Channel_2_Init;       /* ROM */
const char Save_Str[] =  Line1_Save_Init;           /* ROM */
const char Erase_Str1[] =  Line1_Erase_Check;       /* ROM */
const char Erase_Str2_Yes[] =  Line2_Erase_Check_Y; /* ROM */
const char Erase_Str2_No[] =  Line2_Erase_Check_N;  /* ROM */
const char Load_Str[] =  Line1_Load_Init;           /* ROM */
const char Choose_Str[] = Line1_Choose_Str_Init;    /* ROM */
const char Vol_Str[] = Line1_Level_Init;            /* ROM */
const char Vol2_Str[] = Line2_Level_Init;           /* ROM */

const char Channel_Setup_Choice_Display[7][17] =
{
"Band 1 Freq Low ",
"Band 1 Slope Low",
"Band 1 Freq Hi  ",
"Band 1 Slope Hi ",
"Band 1 Delay    ",
"Band 1 Invert   ",
"Band 1 Gain.    "
};						/* ROM */

const char Setup_Choice_Units_Display[7][17] =
{
"        Hz      ",   /* Freq */
"                ",   /* Filter Type */
"        Hz      ",   /* Freq */
"                ",   /* Filter Type */
"      mm        ",   /* Delay mm */
"                ",   /* inverted or not*/
"-      dB       "    /* Attenuation*/
};						/* ROM */

const char Filter_Type_Display[4][17] =
{
"No Filter       ",
"6dB/Octave      ",
"12dB/Octave (BW)",
"24dB/Octave (LR)"
};

const char Inverted_Type_Display[2][17] =
{
"Non Inverted    ",
"Inverted        "
};

const char Choice_Display[6][17] =
{
"Channel 1       ",
"Channel 2       ",
"Parametric EQ   ",
"Volume (atten)  ",
"Save Parameters?",
"Load Parameters?"
};						/* ROM */

const char Para_Filter_Select_Display[2][17] =
{
"Parametric # 1  ",
"Parametric # 2  "
};						/* ROM */

const char Para_Filter_Type_Display[5][17] =
{
"No Filter       ",
"6dB/Octave      ",
"12dB/Octave (BW)",
"24dB/Octave (LR)",
"Parametric      "
};

const char Para_Filter_Line1_Display[4][17] =
{
"Gen Filter 1:   ",
"Filter 1 freq:  ",
"Filter 1 Q:     ",
"Filter 1 Gain:  "
};

const char Para_Filter_Line2_Display[4][17] =
{
"                ",
"       Hz       ",
"                ",
"     dB         "
};

const char Freq_Stepsize_Select_Display[2][17] =
{
"Freq Stepsize   ",
"    Hz          "
};						/* ROM */


const char Yes_No[2][4] =
{
"Yes",
"No "
};


/****************************************************************************/
/* Initialise the PIC							    */
/****************************************************************************/
void Init_IO(void)
{

    int spi_con1, spi_con2;

/* clear IO LOck bit */
    CFGCON = CFGCON  & (0xFFFF - 0x2000);

    
/* We will eventually want to be able to set the following: */
/* IEC0 bit 4, Timer1 */
/* IEC0 bit 9, Timer2 */
/* IEC1 bit 2, USB */
/* IEC1 bit 5, SPI1TX */
//    IEC0 = 0x0000;		// Disable interrupt register 0
//    IEC1 = 0x0000;		// Disable interrupt REGISTER 1
//    IEC2 = 0x0000;		// Disable interrupt REGISTER 2

/* ADCON setup */
    ANSELB = 0x0000;            // A/D Port reconfiguration - dig
    ANSELC = 0x0000;		// A/D Port reconfiguration - dig
    ANSELD = 0x0000;            // A/D Port reconfiguration - dig
    ANSELE = 0x0000;		// A/D Port reconfiguration - dig
    ANSELF = 0x0000;		// A/D Port reconfiguration - dig
    ANSELG = 0x0000;            // A/D Port reconfiguration - dig

/* set peripheral pin select pins*/
/* there are lits of these.      */

/* B0 = PGED1*/
/* B1 = PGEC1*/
/* B2  = REFCLKO */
PPSOutput(3,RPB2,REFCLKO); // Set RB2 pin as output for REFCLK
/* B3  =SPARE */
/* B4  =SPARE */
/* B5  =SPARE */
/* B6  Rotary_Encoder_Select_0 (input)*/
/* B7  Rotary_Encoder_Select_1 (input)*/

/* RB8 - LCD0 (output) */
/* RB9 - LCD1 (output) */
/* RB10 - LCD2 (output) */
//RPB10R = 0x0000;
/* RB11 - LCD3 (output) */
/* RB12 - LCD4 (output) */
/* RB13 - LCD5 (output) */
/* RB14 - LCD6 (output) */
/* RB15 - LCD7 (output) */

/* RC13 = OSCILLATOR */
/* RC14 = OSCILLATOR */
/* RC12 = OSCILLATOR */
/* RC15 = OSCILLATOR */

/* RD0 = SPARE*/
/* RD1 = SPI_CS1*/
/* RD2 = SPI_SCK1*/
/* RD3 = SPI_SDI1 = "0000" = input*/
//SDI1R = 0x0000;
    PPSInput(2,RPD3,SDI1);   //Set RD3 as input for SDI1
    PPSOutput(3,RPD4,SDO1); // Set RD4 pin as output for SDO1

/* RD5 = LCD_RW (output)*/
/* RD6 = LCD_RS (output)*/
/* RD7 = LCD_CS (output)*/

/* RD8 = DSP_RESET*/
/* RD9 = SPI_SS1 - "0111"*/
    PPSOutput(3,RPD9,SS1); // Set RD4 pin as output for SS1

/* RD10 = ANALOGUE_CD*/
/* RD11 = UNUSED (output)*/

/* RE0 = Drives logic 1 as output for test board. */
/* RE1 = MODE_SEL (input)*/
/* RE2 = FN_SEL (input)*/
/* RE3 = PHASEA (input)*/
/* RE4 = PHASEB (input)*/
/* RE5 = SPARE*/
/* RE6 = SPARE*/
/* RE7 = SPARE*/

/* RF0 EEPROM CS\ - active low (output)*/
/* RF1 EEPROM Hold\ - active low (output)*/
/* RF4 UNUSED (output)*/
/* RF5 UNUSED (output)*/

/* RG6 = SPI_SCK2 */
//    PPSOutput(3,RPG6,REFCLKO); // Set RG6 pin as output for REFCLKO

/* RG7 = SPI_SDI2 "0001" */
/* RG8 = SPI_SDO2 = "0110" */
    PPSInput(2,SDI2, RPG7);   //Set Rg7 as input for SDI2
    PPSOutput(1,RPG8,SDO2); // Set Rg8 pin as output for SDO2

/* RG9 = SPI_SS2 = "0001" */
    PPSOutput(4,RPG9,SS2); // Set RG9 pin as output for SS2
    
/* initialise IO */
/* B6,7 = input remainder = output */
/* C = output */
/* D3 = input 0x0008*/
/* E1,2,3,4 = input 0x001E*/
/* F = output */
/* G7 = input 0X0080*/
        TRISB=0X00C0;
        TRISC=0X0000;
        TRISD=0X0008;
        TRISE=0x001E;
        TRISF=0x0000;
        TRISG=0x0080;

/* Set outputs to normal logic */
        ODCB = 0x0000;
        ODCC = 0x0000;
        ODCD = 0x0000;
        ODCE = 0x0000;
        ODCF = 0x0000;
        ODCG = 0x0000;

/* Write to ports */
/* need to update these to appropriate values */
        PORTB = 0x0000;
        PORTC = 0x0000;
        PORTD = 0x0000;
        PORTE = 0x0001;  /* drive logic high to E0*/
        PORTF = 0x0000;
        PORTG = 0x0000;

/* Change Notice Enable = OFF*/
        CNCONB = 0X0000;
        CNCONC = 0X0000;
        CNCOND = 0X0000;
        CNCONE = 0X0000;
        CNCONF = 0X0000;
        CNCONG = 0X0000;


/* Hold DSP in RESET */
SetDSPReset;        /* Hold DSP Reset Low  */
SetDACReset;	    /* Hold DAC in RESET */
PORTB = 0;   

CselClearEEPROM; /* Clear EEPROM Chip Select*/
HoldClearEEPROM; /* Clear EEPROM hold signal */

/* Initialise the SPI interface */
/*
 * Channel 1
 * Master
 * Positive Pol
 * No slave enable
 * Default clock edge
 * Sample phase control
 * 8 bit character mode (dont use 16 or 32)
 * SDO pin enable (default)
 * No idle functionality
 * No debug
 * Turn SPI On
 * No frame
 *
 * fpbDiv 64
*/
SpiChnOpen(1,SPICON_MSTEN | SPICON_CKP | SPICON_ON, 256);

/* Configure Reference Clock Output to 12MHz.  */
mOSCREFOTRIMSet(REFTRIM);
OSCREFConfig(OSC_REFOCON_USBPLL, //USB-PLL clock output used as REFCLKO source
            OSC_REFOCON_OE | OSC_REFOCON_ON, //Enable and turn on the REFCLKO
            RODIV_PROG);

 //Configure SPI in I2S mode with 24-bit stereo audio.
 spi_con1 = SPI_OPEN_MSTEN | //Master mode enable
        SPI_OPEN_SSEN | //Enable slave select function
        SPI_OPEN_CKP_HIGH | //Clock polarity Idle High Actie Low
        SPI_OPEN_ENHBUF | // Enhanced Mode
        SPI_OPEN_TBE_HALF_EMPTY | //generate event when TX buffer more than half empty
        SPI_OPEN_MODE16 | //Data mode: 24b
        SPI_OPEN_MODE32 | //Data mode: 24b
        SPI_OPEN_MCLKSEL | //Clock selected is reference clock
        SPI_OPEN_FSP_HIGH; //Frame Sync Pulse is active high

 spi_con2 = SPI_OPEN2_AUDEN | //Enable Audio mode
        SPI_OPEN2_AUDMOD_I2S; //Enable I2S mode

//Configure and turn on the SPI2 module.
SpiChnOpenEx(SPI_CHANNEL2, spi_con1, spi_con2, (mclk / baud_rate));

     
//Configure SPI in I2S mode with 24-bit stereo audio.
 spi_con1 = SPI_OPEN_MSTEN | //Master mode enable
        SPI_OPEN_SSEN | //Enable slave select function
        SPI_OPEN_CKP_HIGH | //Clock polarity Idle High Actie Low
        SPI_OPEN_ENHBUF | // Enhanced Mode
        SPI_OPEN_TBE_HALF_EMPTY | //generate event when TX buffer more than half empty
        SPI_OPEN_MODE16 | //Data mode: 24b
        SPI_OPEN_MODE32 | //Data mode: 24b
        SPI_OPEN_MCLKSEL | //Clock selected is reference clock
        SPI_OPEN_FSP_HIGH; //Frame Sync Pulse is active high

 spi_con2 = SPI_OPEN2_AUDEN | //Enable Audio mode
        SPI_OPEN2_AUDMOD_I2S; //Enable I2S mode
//Configure and turn on the SPI1 module.
//SpiChnOpenEx(SPI_CHANNEL1, spi_con1, spi_con2, (mclk / baud_rate));

 SPI2CON = SPI2CON & 0xffff7fff; // turn things off
 SPI2STATCLR = 1<<6;


Data_Values.Ch_1_FL_Sel = CH_1_FL_Default;       /* Freq is in Hz*/
Data_Values.Ch_1_SL_Sel = CH_1_SL_Default;     /* Lower Filter slope (none, 6, 12 and 24)*/
Data_Values.Ch_1_FH_Sel = CH_1_FH_Default;     /* Freq is in Hz*/
Data_Values.Ch_1_SH_Sel = CH_1_SH_Default;     /* Lower Filter slope (none, 6, 12 and 24)*/
Data_Values.Ch_1_Inv_Sel = CH_1_Inv_Default ;    /* Inverted*/
Data_Values.Ch_1_Del_Sel = CH_1_Del_Default;    /* Delay in mm*/
Data_Values.Ch_1_Att_Sel = CH_1_Att_Default;    /* Attenuation in tenths of dB*/

Data_Values.Ch_2_FL_Sel  = CH_2_FL_Default;     /* Freq is in Hz*/
Data_Values.Ch_2_SL_Sel = CH_2_SL_Default;     /* Lower Filter slope (none, 6, 12 and 24)*/
Data_Values.Ch_2_FH_Sel = CH_1_FH_Default;     /* Freq is in Hz*/
Data_Values.Ch_2_SH_Sel = CH_1_SH_Default;     /* Lower Filter slope (none, 6, 12 and 24)*/
Data_Values.Ch_2_Inv_Sel = CH_2_Inv_Default;    /* Inverted*/
Data_Values.Ch_2_Del_Sel = CH_2_Del_Default;    /* Delay in mm*/
Data_Values.Ch_2_Att_Sel = CH_2_Att_Default;    /* Attenuation in tenths of dB*/

Data_Values.G1_Type_Sel = G1_Type_Default;     /* G1 types are: none, LP6, LP12, Para*/
Data_Values.G1_F_Sel  = G1_Freq_Default;        /* Freq is in Hz*/
Data_Values.G1_Q_Sel  = G1_Q_Default;        /* Q is in setps of 0.1*/
Data_Values.G1_Gain_Sel  = G1_Gain_Default;     /* Gain is in tenths of dB*/
Data_Values.G2_Type_Sel = G2_Type_Default;     /* G1 types are: none, LP6, LP12, Para*/
Data_Values.G2_F_Sel = G2_Freq_Default;        /* Freq is in Hz*/
Data_Values.G2_Q_Sel  = G2_Q_Default;        /* Q is in setps of 0.1*/
Data_Values.G2_Gain_Sel  = G2_Gain_Default;     /* Gain is in tenths of dB*/

Data_Values.Volume_Gain_Sel = Volume_Gain_Default; /* volume attenuation in dB */
}


/*
Data_Values.Ch_1_FL_Sel);   Data_Values.Ch_2_FL_Sel);
Data_Values.Ch_1_SL_Sel);   Data_Values.Ch_2_SL_Sel);
Data_Values.Ch_1_FH_Sel);   Data_Values.Ch_2_FH_Sel);
Data_Values.Ch_1_SH_Sel);   Data_Values.Ch_2_SH_Sel);
Data_Values.Ch_1_Inv_Sel);  Data_Values.Ch_2_Inv_Sel);
Data_Values.Ch_1_Del_Sel);  Data_Values.Ch_2_Del_Sel);
Data_Values.Ch_1_Att_Sel);  Data_Values.Ch_2_Att_Sel);

Data_Values.G1_F_Sel);
Data_Values.G1_Type_Sel);
Data_Values.G1_Q_Sel);
Data_Values.G1_Gain_Sel);
Data_Values.G2_F_Sel);
Data_Values.G2_Type_Sel);
Data_Values.G2_Q_Sel);
Data_Values.G2_Gain_Sel);

Data_Values.Volume_Gain_Sel);
*/

/****************************************************************************/
/* Do this in one place  -  save memory and hassles with consistency!    */
/****************************************************************************/
void Load_From_Memory(char Mem_Bank)
{
unsigned int Data_Addr;          /* temp address counter */

/* Select EEPROM device */
CselEEPROM;
/* Read in the parameters for crossover points from default bank of EEPROM */

/* This is the base address of the channel's data in EEPROM*/
Data_Addr = (int)Mem_Bank * ParmSet_Array_Size;


/* CH_1_Freq Freq_Sel at Freq_Offset */
HDWordReadSPI((Data_Addr + FL_Offset), &(Data_Values.Ch_1_FL_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_FL_Sel < Freq_Min)
    Data_Values.Ch_1_FL_Sel = Freq_Min;
else if (Data_Values.Ch_1_FL_Sel > Freq_Max)
    Data_Values.Ch_1_FL_Sel = Freq_Max;

/* CH_1_Slope Freq_Sel at SL_Offset */
HDWordReadSPI((Data_Addr + SL_Offset), &(Data_Values.Ch_1_SL_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_SL_Sel < Filter_Min)
    Data_Values.Ch_1_SL_Sel = Filter_Min;
else if (Data_Values.Ch_1_SL_Sel > Filter_Max)
    Data_Values.Ch_1_SL_Sel = Filter_Max;

/* CH_1_FH Sel Freq_Sel at FH_Offset */
HDWordReadSPI((Data_Addr + FH_Offset), &(Data_Values.Ch_1_FH_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_FH_Sel < Freq_Min)
    Data_Values.Ch_1_FH_Sel = Freq_Min;
else if (Data_Values.Ch_1_FH_Sel > Freq_Max)
    Data_Values.Ch_1_FH_Sel = Freq_Max;

/* CH_1_Slope Hiigh  at SH_Offset */
HDWordReadSPI((Data_Addr + SH_Offset), &(Data_Values.Ch_1_SH_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_SH_Sel < Filter_Min)
    Data_Values.Ch_1_SH_Sel = Filter_Min;
else if (Data_Values.Ch_1_SH_Sel > Filter_Max)
    Data_Values.Ch_1_SH_Sel = Filter_Max;

/* CH_1_Inverted  at Inv_Offset */
HDWordReadSPI((Data_Addr + Inv_Offset), &(Data_Values.Ch_1_Inv_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Inv_Sel < CHInverted)
    Data_Values.Ch_1_Inv_Sel = CHInverted;
else if (Data_Values.Ch_1_Inv_Sel > CHNonInverted)
    Data_Values.Ch_1_Inv_Sel = CHNonInverted;

/* CH_1_Delay  at Del_Offset */
HDWordReadSPI((Data_Addr + Del_Offset), &(Data_Values.Ch_1_Del_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Del_Sel < Del_Min)
    Data_Values.Ch_1_Del_Sel = Del_Min;
else if (Data_Values.Ch_1_Del_Sel > Del_Max)
    Data_Values.Ch_1_Del_Sel = Del_Max;

/* CH_1_Att  at Att_Offset */
HDWordReadSPI((Data_Addr + Att_Offset), &(Data_Values.Ch_1_Att_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Att_Sel < Att_Min)
    Data_Values.Ch_1_Att_Sel = Att_Min;
else if (Data_Values.Ch_1_Att_Sel > Att_Max)
    Data_Values.Ch_1_Att_Sel = Att_Max;


/* This is the base address of the second channel's data in EEPROM*/
Data_Addr = Mem_Bank * ParmSet_Array_Size + Channel_Set_Array_Size;

/* CH_2_Freq Freq_Sel at Freq_Offset */
HDWordReadSPI((Data_Addr + FL_Offset), &(Data_Values.Ch_2_FL_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_FL_Sel < Freq_Min)
    Data_Values.Ch_2_FL_Sel = Freq_Min;
else if (Data_Values.Ch_2_FL_Sel > Freq_Max)
    Data_Values.Ch_2_FL_Sel = Freq_Max;

/* CH_2_Slope Freq_Sel at SL_Offset */
HDWordReadSPI((Data_Addr + SL_Offset), &(Data_Values.Ch_2_SL_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_SL_Sel < Filter_Min)
    Data_Values.Ch_2_SL_Sel = Filter_Min;
else if (Data_Values.Ch_2_SL_Sel > Filter_Max)
    Data_Values.Ch_2_SL_Sel = Filter_Max;

/* CH_2_FH Sel Freq_Sel at FH_Offset */
HDWordReadSPI((Data_Addr + FH_Offset), &(Data_Values.Ch_2_FH_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_FH_Sel < Freq_Min)
    Data_Values.Ch_2_FH_Sel = Freq_Min;
else if (Data_Values.Ch_2_FH_Sel > Freq_Max)
    Data_Values.Ch_2_FH_Sel = Freq_Max;

/* CH_2_Slope High  at SH_Offset */
HDWordReadSPI((Data_Addr + SH_Offset), &(Data_Values.Ch_2_SH_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_SH_Sel < Filter_Min)
    Data_Values.Ch_2_SH_Sel = Filter_Min;
else if (Data_Values.Ch_2_SH_Sel > Filter_Max)
    Data_Values.Ch_2_SH_Sel = Filter_Max;

/* CH_2_Inverted  at Inv_Offset */
HDWordReadSPI((Data_Addr + Inv_Offset), &(Data_Values.Ch_2_Inv_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Inv_Sel < CHInverted)
    Data_Values.Ch_2_Inv_Sel = CHInverted;
else if (Data_Values.Ch_2_Inv_Sel > CHNonInverted)
    Data_Values.Ch_2_Inv_Sel = CHNonInverted;

/* CH_2_Delay  at Del_Offset */
HDWordReadSPI((Data_Addr + Del_Offset), &(Data_Values.Ch_2_Del_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Del_Sel < Del_Min)
    Data_Values.Ch_2_Del_Sel = Del_Min;
else if (Data_Values.Ch_2_Del_Sel > Del_Max)
    Data_Values.Ch_2_Del_Sel = Del_Max;

/* CH_2_Att  at Att_Offset */
HDWordReadSPI((Data_Addr + Att_Offset), &(Data_Values.Ch_2_Att_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Att_Sel < Att_Min)
    Data_Values.Ch_2_Att_Sel = Att_Min;
else if (Data_Values.Ch_2_Att_Sel > Att_Max)
    Data_Values.Ch_2_Att_Sel = Att_Max;


/* This is the base address of the second channel's data in EEPROM*/
Data_Addr = Mem_Bank * ParmSet_Array_Size;

/* Gen Filter Parameters*/
HDWordReadSPI((Data_Addr + G1_F_Offset), &(Data_Values.G1_F_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.G1_F_Sel < Freq_Min)
    Data_Values.G1_F_Sel = Freq_Min;
else if (Data_Values.G1_F_Sel > Freq_Max)
    Data_Values.G1_F_Sel = Freq_Max;

/* Gen Filter Type Parameters*/
HDWordReadSPI((Data_Addr + G1_Type_Offset), &(Data_Values.G1_Type_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.G1_Type_Sel < NoFilter)
    Data_Values.G1_Type_Sel = NoFilter;
else if (Data_Values.G1_Type_Sel > G_Filter_Max)
    Data_Values.G1_Type_Sel = G_Filter_Max;

/* Gen Filter Q Parameters*/
HDWordReadSPI((Data_Addr + G1_Q_Offset), &(Data_Values.G1_Q_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.G1_Q_Sel < Q_Min)
    Data_Values.G1_Q_Sel = Q_Min;
else if (Data_Values.G1_Q_Sel > Q_Max)
    Data_Values.G1_Q_Sel = Q_Max;

/* Gen Filter Gain Parameters*/
HDWordReadSPI((Data_Addr + G1_Gain_Offset), &(Data_Values.G1_Gain_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.G1_Gain_Sel < G_Gain_Min)
    Data_Values.G1_Gain_Sel = G_Gain_Min;
else if (Data_Values.G1_Gain_Sel > G_Gain_Max)
    Data_Values.G1_Gain_Sel = G_Gain_Max;


/* Gen Filter Parameters*/
HDWordReadSPI((Data_Addr + G2_F_Offset), &(Data_Values.G2_F_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.G2_F_Sel < Freq_Min)
    Data_Values.G2_F_Sel = Freq_Min;
else if (Data_Values.G2_F_Sel > Freq_Max)
    Data_Values.G2_F_Sel = Freq_Max;

/* Gen Filter Type Parameters*/
HDWordReadSPI((Data_Addr + G2_Type_Offset), &(Data_Values.G2_Type_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.G2_Type_Sel < NoFilter)
    Data_Values.G2_Type_Sel = NoFilter;
else if (Data_Values.G2_Type_Sel > G_Filter_Max)
    Data_Values.G2_Type_Sel = G_Filter_Max;

/* Gen Filter Q Parameters*/
HDWordReadSPI((Data_Addr + G2_Q_Offset), &(Data_Values.G2_Q_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.G2_Q_Sel < Q_Min)
    Data_Values.G2_Q_Sel = Q_Min;
else if (Data_Values.G2_Q_Sel > Q_Max)
    Data_Values.G2_Q_Sel = Q_Max;

/* Gen Filter Gain Parameters*/
HDWordReadSPI((Data_Addr + G2_Gain_Offset), &(Data_Values.G2_Gain_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.G2_Gain_Sel < G_Gain_Min)
    Data_Values.G2_Gain_Sel = G_Gain_Min;
else if (Data_Values.G2_Gain_Sel > G_Gain_Max)
    Data_Values.G2_Gain_Sel = G_Gain_Max;

/* Attenuation Gain Parameters*/

HDWordReadSPI((Data_Addr + Volume_Gain_Offset), &(Data_Values.Volume_Gain_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Volume_Gain_Sel < Volume_Min)
    Data_Values.Volume_Gain_Sel = Volume_Min;
else if (Data_Values.Volume_Gain_Sel > Volume_Max)
    Data_Values.Volume_Gain_Sel = Volume_Max;


/* deselect EEPROM*/
CselClearEEPROM;
}


/************************************************************************/
/* 	Write Line 1 and Line 2 of LCD					*/
/************************************************************************/
void Update_LCD(void)
{

/* Set Line 1 Address */
Write_Control_LCD_CSEL;
LCD_Data((LCD_Line_1));
DelayUs(LCD_Settle_Before_Write);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Delay_After_Write);
Write_Control_LCD_CSEL;

DelayMs(LCD_Settle_After_Control);

/*set the pointers at the line array adddresses */
line1_ptr = &(line1[0]);
for (loop = 0; loop < 16; loop++)
	{
		/* Write string 1 to LCD */
		Write_Data_Enable_LCD_CSEL;
		LCD_Data((char)*line1_ptr);
		DelayUs(LCD_Settle_Before_Write);
                Write_Data_LCD_CSEL;
		DelayUs(LCD_Delay_After_Write);
		Write_Data_Enable_LCD_CSEL;
		line1_ptr++;
	}


/* Set Line 2 Address */
Write_Control_LCD_CSEL;
LCD_Data((LCD_Line_2));
DelayUs(LCD_Settle_Before_Write);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Delay_After_Write);
Write_Control_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);

/*set the pointers at the line array adddresses */
line2_ptr = &(line2[0]);
for (loop = 0; loop < 16; loop++)
	{
		/* Write string 2 to LCD */
		Write_Data_Enable_LCD_CSEL;
		LCD_Data((char)*line2_ptr);
         	DelayUs(LCD_Settle_Before_Write);
		Write_Data_LCD_CSEL;
		DelayUs(LCD_Delay_After_Write);
		Write_Data_Enable_LCD_CSEL;
		line2_ptr++;
	}

}


/************************************************************************/
/* 						Init  			*/
/*  This is for a standard 16 x 2 character LCD... 			*/
/************************************************************************/
void Init_LCD(void)
{
/* Function Select */
Write_Control_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
LCD_Data(LCD_Function);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_LCD_CSEL;

DelayUs(LCD_Delay_After_Write);

/* Display On */
LCD_Data(LCD_Display_On);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);

/* Display Clear */
LCD_Data(LCD_Display_Clear);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);

/* Entry Mode */
LCD_Data(LCD_Display_Entry_Mode);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
DelayUs(LCD_Settle_Before_Write);
Write_Control_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);

/* Display Clear */
Write_Control_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
LCD_Data(LCD_Display_Clear);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_Enable_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);
Write_Control_Enable_LCD_CSEL;

DelayMs(LCD_Settle_After_Control);
}




/************************************************************************/
/* Display relevant data on the LCD 			
Need to build display lies for:
Display_Fn = 
	Choosing_Fn	
	Volume_Mode
	XO_Mode 
	Save_Mode
	Load_Mode
	EQ_Mode
	Change_Mode							*/
/*
and the following explicit functions
	XO_Band_Sel_Fn
	XO_Parm_Sel
	XO_LF_Parm_Fn
	XO_HF_Parm_Fn
	XO_SL_Parm_Fn
	XO_SU_Parm_Fn
	XO_Del_Parm_Fn
	XO_Att_Parm_Fn

	Save_Sel_Fn

	Load_Sel_Fn							*/

/*
#define EQ_Sel_Fn 				
#define EQ_Freq_Sel_Fn 			
#define EQ_Q_Sel_Fn  	
#define EQ_Gain_Sel_Fn 	
#define EQ_Type_Sel_Fn 	
									*/
/************************************************************************/
void Display_Data(int Display_Fn, int Display_Value)
{
char loop;
int Display_Value_Temp;
int Display_Ptr_Offset;  /* used to reference lookup into display string*/

/* Use this to remember if the Display Value needs a - sign */
Display_Value_Temp = Display_Value;

/* Display Pointer Offset allows reuse of the strings and code for */
/* Channel 1 and 2 */
if((Display_Fn >= Ch_2_FL) && (Display_Fn <= Ch_2_Att))
    Display_Ptr_Offset = (Ch_2_FL - Ch_1_FL);
else
    Display_Ptr_Offset = 0;


/* set the pointers at the line array adddresses */
/* The idea here is to use pointers into the Line1 and Line 2 arrays */
/* The code efficienct using direct array operations is abysmal.     */
/* Remember that this fuction builds the lines from right to left    */
/* You did the because it makes it easer to build the numbers....    */
/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;


/********************************************************************/
/* 					Build Line 1 string in here */
/********************************************************************/

/************************************************************************/
/* Choosing Value - user selecting the fuinction to modify              */
/************************************************************************/
	if (Display_Fn == Choosing_Fn)
		{
		*line1_ptr = Choose_Str[loop];
		*line2_ptr = Choice_Display[Display_Value - 1][loop];					
		}

/************************************************************************/
/* Save Function display - for Normal Save functionality        	*/
/************************************************************************/
        else if ((Display_Fn == Save_Sel_Fn) && (Display_Value <= Max_Mem_Banks))
		{
		/* Build string */
		*line1_ptr = Save_Str[loop];

		/* Convert 0-3 integer to 1-3 ASCII for bank number */
		if (loop == 13)
			*line1_ptr = Display_Value + 49;
		}
/************************************************************************/
/* Save Function display - for Erasing EEPROM                   	*/
/************************************************************************/
        else if ((Display_Fn == Save_Sel_Fn) && (Display_Value > Max_Mem_Banks))
		{
		/* Build string */
		*line1_ptr = Erase_Str1[loop];
                if (Display_Value == Mem_Banks_Do_Not_Erase)
                    *line2_ptr = Erase_Str2_No[loop];
                else if (Display_Value == Mem_Banks_Erase)
                    *line2_ptr = Erase_Str2_Yes[loop];

		/* Convert display Vaue into "Yes or No" */
		if ((loop == 14) && (Display_Value == Mem_Banks_Do_Not_Erase))
			*line1_ptr = 'N';
                else if ((loop == 14) && (Display_Value == Mem_Banks_Erase))
			*line1_ptr = 'Y';
		}


/************************************************************************/
/*Load function display							*/
/************************************************************************/
	else if (Display_Fn == Load_Sel_Fn)
		{
		/* Build string */
		*line1_ptr = Load_Str[loop];

		/* Convert 0-3 integer to 1-3 ASCII for bank number */
		if (loop == 15)
			*line1_ptr = Display_Value + 49;
		}


/************************************************************************/
/* Volume Select Function display                                       */
/************************************************************************/
        else if ((Display_Fn == Volume_Sel_Fn))
		{
                if (Display_Value < 0)
                    Display_Value = -Display_Value;
		/* Build string */
		*line1_ptr = Choice_Display[Volume_Sel_Fn_Choice_Display_Row][loop];
                *line2_ptr = Setup_Choice_Units_Display[Volume_Sel_Fn_Units_Display_Row][loop];

          	if ((loop == 3))
		{
                    *line2_ptr = '.'; 
		}

                /* Put up Frequency */
                else if (loop == 4)
                {
                    if (Display_Value & 0x01)
                    {
                        *line2_ptr = '5';
                    }
                    else
                    {
                        *line2_ptr = '0';
                    }
                    Display_Value = Display_Value >> 1;
                }
                else if ((loop < 3) && (loop > 0))
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value != 0)
			    *line2_ptr = (display_temp)+48;
                        else if ((Display_Value == 0) && (loop == 2))
			    *line2_ptr = '0';
			else
			    *line2_ptr = Blank_Char;
			Display_Value = (unsigned int)Display_Value/10;
                    }
                else;
		}


/************************************************************************/
/* Ch_1 & 2 lower and upper Frequency Selection Function display                           */
/************************************************************************/
        else if ((Display_Fn == Ch_1_FL) || (Display_Fn == Ch_2_FL) ||
                  (Display_Fn == Ch_1_FH) || (Display_Fn == Ch_2_FH))
		{
		/* Build string */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Ch_1_FL - Display_Ptr_Offset][loop];
                *line2_ptr = Setup_Choice_Units_Display[Display_Fn - Ch_1_FL - Display_Ptr_Offset][loop];

                if (((Display_Fn == Ch_2_FL) || (Display_Fn == Ch_2_FH) )&& (loop == 5))
                    *line1_ptr = '2';
                else;

   
          	if ((loop == 2) && (Display_Value > 0))
		{
                    *line2_ptr = 44;  /* 44 is "," */
		}

                /* Put up Frequency */
                else if (loop < 6)
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value > 0)
			    *line2_ptr = (display_temp)+48;
			else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
		}

/************************************************************************/
/* Ch_1 Freequency Speed Select display                                 */
/************************************************************************/
        else if ((Display_Fn == Freq_Speed_Counter_1_fn) || (Display_Fn == Freq_Speed_Counter_2_fn))
		{
		/* Build string */
		*line1_ptr = Freq_Stepsize_Select_Display[0][loop];
                *line2_ptr = Freq_Stepsize_Select_Display[1][loop];

                /* Put up Frequency */
                if (loop <= 2)
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value > 0)
			    *line2_ptr = (display_temp)+48;
			else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
		}

/************************************************************************/
/* Ch_1 & 2 lower and upper Frequency Selection Function display                           */
/************************************************************************/
        else if ((Display_Fn == Ch_1_SL) || (Display_Fn == Ch_2_SL) ||
                  (Display_Fn == Ch_1_SH) || (Display_Fn == Ch_2_SH))
		{
		/* Build string */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Ch_1_FL - Display_Ptr_Offset][loop];
                *line2_ptr = Filter_Type_Display[Display_Value - Filter_Min][loop];

                if (((Display_Fn == Ch_2_SL) || (Display_Fn == Ch_2_SH) )&& (loop == 5))
                    *line1_ptr = '2';
                }

/************************************************************************/
/* Ch_1 & 2 Delay Selection Function display                           */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Del) || (Display_Fn == Ch_2_Del))
		{
		/* Build string */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Ch_1_FL - Display_Ptr_Offset][loop];
                *line2_ptr = Setup_Choice_Units_Display[Display_Fn - Ch_1_FL - Display_Ptr_Offset][loop];

                if ((Display_Fn == Ch_2_Del) && (loop == 5))
                    *line1_ptr = '2';

          	if ((loop == 2) && (Display_Value > 0))
		{
                    *line2_ptr = 44;  /* 44 is "," */
		}

                /* Put up Frequency */
                else if (loop < 6)
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value > 0)
			    *line2_ptr = (display_temp)+48;
			else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
		}

/************************************************************************/
/* Ch_1 & 2 Invert Selection Function display                           */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Inv) || (Display_Fn == Ch_2_Inv))
		{
		/* Build string */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Ch_1_FL - Display_Ptr_Offset][loop];
                if(Display_Value == CHInverted)
                    *line2_ptr = Inverted_Type_Display[1][loop];
                else
                    *line2_ptr = Inverted_Type_Display[0][loop];
 
                if ((Display_Fn == Ch_2_Inv) && (loop == 5))
                    *line1_ptr = '2';
            }
/************************************************************************/
/* Ch_1 & 2 Attenuation Selection Function display                           */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Att) || (Display_Fn == Ch_2_Att))
		{
		/* Build string */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Ch_1_FL - Display_Ptr_Offset][loop];
                *line2_ptr = Setup_Choice_Units_Display[Display_Fn - Ch_1_FL - Display_Ptr_Offset][loop];

                if ((Display_Fn == Ch_2_Att) && (loop == 5))
                    *line1_ptr = '2';

                if (Display_Value < 0)
                {
                    Display_Value = -Display_Value;
                }

                if (loop == 3)
                {
                   *line2_ptr = '.';
                }
                /* Put up attenuation */
                else if ((loop < 5) && (loop > 0))
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value != 0)
			    *line2_ptr = (display_temp)+48;
                        else if ((Display_Value == 0) && ((loop == 2) || (loop == 4)))
			    *line2_ptr = '0';
                        else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
                else if (loop == 0)
                    {
                        if (Display_Value_Temp < 0 )
                            *line2_ptr = '-';
                        else
                            *line2_ptr = '+';
                    }
                else;
                
		}
/************************************************************************/
/* Gen1 and Gen2 filter Selection Function display                      */
/************************************************************************/
        else if ((Display_Fn == G1_Type) || (Display_Fn == G2_Type))
		{
		/* Build string */
                if(Display_Fn == G1_Type)
                    *line1_ptr = Para_Filter_Line1_Display[Display_Fn - G1_Type][loop];
                else
                    *line1_ptr = Para_Filter_Line1_Display[Display_Fn - G2_Type][loop];
                *line2_ptr = Para_Filter_Type_Display[Display_Value - Filter_Min][loop];

                if ((Display_Fn == G2_Type) && (loop == 11))
                    {
                        *line1_ptr = '2';
                   }
                }
/************************************************************************/
/* Gen1 and Gen2 filter Frequency Function display                      */
/************************************************************************/
        else if ((Display_Fn == G1_Freq) || (Display_Fn == G2_Freq))
		{
		/* Build string */
                if(Display_Fn == G1_Freq)
                {
                    *line1_ptr = Para_Filter_Line1_Display[Gen_Freq_Display_Line][loop];
                    *line2_ptr = Para_Filter_Line2_Display[Display_Fn - G1_Type][loop];
                }
                else
                {
                    *line1_ptr = Para_Filter_Line1_Display[Gen_Freq_Display_Line][loop];
                    *line2_ptr = Para_Filter_Line2_Display[Display_Fn - G2_Type][loop];
                }

                if ((Display_Fn == G2_Freq) && (loop == 7))
                {
                    *line1_ptr = '2';
                }

          	if ((loop == 2) && (Display_Value > 0))
		{
                    *line2_ptr = 44;  /* 44 is "," */
		}

                /* Put up Frequency */
                else if (loop < 6)
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value > 0)
			    *line2_ptr = (display_temp)+48;
			else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
		}
/************************************************************************/
/* Gen1 and Gen2 Q Selection Function display                           */
/************************************************************************/
        else if ((Display_Fn == G1_Q) || (Display_Fn == G2_Q))
		{
		/* Build string */
                if(Display_Fn == G1_Q)
                {
                    *line1_ptr = Para_Filter_Line1_Display[Gen_Q_Display_Line][loop];
                    *line2_ptr = Para_Filter_Line2_Display[Display_Fn - G1_Type][loop];
                }
                else
                {
                    *line1_ptr = Para_Filter_Line1_Display[Gen_Q_Display_Line][loop];
                    *line2_ptr = Para_Filter_Line2_Display[Display_Fn - G2_Type][loop];
                }

                if ((Display_Fn == G2_Q) && (loop == 7))
                {
                    *line1_ptr = '2';
                }

          	if ((loop == 2))
		{
                    *line2_ptr = 46;  /* 46 is .," */
		}

                /* Put up Frequency */
                else if (loop < 4)
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value > 0)
			    *line2_ptr = (display_temp)+48;
			else if ((loop == 3) || (loop == 1))
                            *line2_ptr = '0';
                        else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
		}
/************************************************************************/
/* Gen1 and Gen2 Gain Selection Function display                        */
/************************************************************************/
        else if ((Display_Fn == G1_Gain) || (Display_Fn == G2_Gain))
                {
		/* Build string */
                if(Display_Fn == G1_Gain)
                {
                    *line1_ptr = Para_Filter_Line1_Display[Gen_Gain_Display_Line][loop];
                    *line2_ptr = Para_Filter_Line2_Display[Display_Fn - G1_Type][loop];
                }
                else
                {
                    *line1_ptr = Para_Filter_Line1_Display[Gen_Gain_Display_Line][loop];
                    *line2_ptr = Para_Filter_Line2_Display[Display_Fn - G2_Type][loop];
                }

                if ((Display_Fn == G2_Gain) && (loop == 7))
                {
                    *line1_ptr = '2';
                }

                if (Display_Value < 0)
                    Display_Value = -Display_Value;

          	if (loop == 0)
                    {
                        if (Display_Value_Temp < 0)
                            *line2_ptr = '-'; 
                        else
                            *line2_ptr = '+'; 
                    }
                else if (loop == 3)
                    {
                        *line2_ptr = 46;  /* 46 is "." */
                    }
                else if ((loop == 4) || (loop < 3))
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value > 0)
			    *line2_ptr = (display_temp)+48;
                        else if ((Display_Value == 0) && ((loop == 4)||(loop == 2)))
			    *line2_ptr = '0';
                        else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
                }

	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 							*/
	/************************************************************************/
Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}


//Print_Data(int Number_of_Words, char *ptr_to_Data)
void Print_Data(int Number_of_Words, unsigned char *ptr_to_Data, unsigned int Address_to_Print)
{
char loop;
unsigned int display_temp, display_data;
unsigned int Address_Temp, Address_Temp_Hex;

	/*set the pointers at the line array adddresses */
	/* The idea here is to use pointers into the Line1 and Line 2 arrays */
	/* The code efficiency using direct array operations is abysmal.     */
	/* Remember that this fuction builds the lines from right to left    */
	/* You did the because it makes it easer to build the numbers....    */
	/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
display_data = *ptr_to_Data;    /* initial dtaa to diaplay */
Address_Temp = Address_to_Print;
Address_Temp_Hex = Address_to_Print;

for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = display_data % 16;
	if (loop > (15 - 2*Number_of_Words))
		{
			if (display_temp <= (unsigned char)9)
				*line1_ptr = (display_temp)+48;
			else
				*line1_ptr = (display_temp)+55;
		}
	display_data = display_data>>4;
	if (!(loop % 2)  && (((15 - loop)/2) < Number_of_Words))  /* then need to read next byte */
		{
		ptr_to_Data++;
		display_data = *ptr_to_Data;
		}

	display_temp = Address_Temp % 10;
	if ((Address_Temp > (unsigned int)0) || (loop > 14))
		*line2_ptr = (display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Address_Temp = Address_Temp/10;


	if ((loop <= 8) && (loop > 4))
		{
			display_temp = Address_Temp_Hex % 16;
			if (display_temp <= (unsigned char)9)
				*line2_ptr = (display_temp)+48;
			else
				*line2_ptr = (display_temp)+55;
			Address_Temp_Hex = Address_Temp_Hex>>4;
		}


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 							*/
	/************************************************************************/

Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}


void Display_Debug_Data(int Display_Value)
{
char loop;
int Display_Value_Temp, Display_Value_Hold;
		/* Use this to remember if the Display Value needs a - sign */
Display_Value_Hold = Display_Value;
if (Display_Value_Hold <0)
{
   Display_Value = - Display_Value;
}
Display_Value_Temp = Display_Value;

                /*set the pointers at the line array adddresses */
		/* The idea here is to use pointers into the Line1 and Line 2 arrays */
		/* The code efficienct using direct array operations is abysmal.     */
		/* Remember that this fuction builds the lines from right to left    */
		/* You did the because it makes it easer to build the numbers....    */
		/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = Display_Value_Temp % 16;
	if ((Display_Value_Temp != 0) || (loop > 7))
		{
			if (display_temp <= 9)
				*line1_ptr = ((unsigned char)display_temp)+48;
			else
				*line1_ptr = ((unsigned char)display_temp)+55;
		}
	else
		*line1_ptr = Blank_Char;
	Display_Value_Temp = Display_Value_Temp/16;


	display_temp = Display_Value % 10;
	if ((Display_Value != 0))
		*line2_ptr = ((unsigned char)display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Display_Value = Display_Value/10;

        if ((loop == 1)  && (Display_Value_Hold < 0))
        {
            *line1_ptr = '-';
            *line2_ptr = '-';
        }
	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 							*/
	/************************************************************************/

Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}


void Display_Debug_Data_Char(unsigned char Display_Value)
{
char loop;
int Display_Value_Temp;

/* Use this to remember if the Display Value needs a - sign */
Display_Value_Temp = Display_Value;

/*set the pointers at the line array adddresses */
/* The idea here is to use pointers into the Line1 and Line 2 arrays */
/* The code efficienct using direct array operations is abysmal.     */
/* Remember that this fuction builds the lines from right to left    */
/* You did the because it makes it easer to build the numbers....    */
/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = (unsigned char) Display_Value_Temp % 16;
	if ((Display_Value_Temp > 0) || (loop > 13))
		{
			if (display_temp <= 9)
				*line1_ptr = (unsigned char) (display_temp)+48;
			else
				*line1_ptr = (unsigned char) (display_temp)+55;
		}
	else
		*line1_ptr = Blank_Char;
	Display_Value_Temp = (unsigned char) Display_Value_Temp/16;


	display_temp = (unsigned char)Display_Value % 10;
	if ((Display_Value > (unsigned char)0) || (loop > (char)13))
		*line2_ptr = (unsigned char) (display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Display_Value = (unsigned char) Display_Value/10;


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 							*/
	/************************************************************************/
Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}


/******************************************************************************/
/*       this function updates filters in a sensible manner                   */
/* The __Optimise__ attribute is there to stop the do - while function being  */
/* reordered - the damn thing does not work if the function is optimised!!!   */
/******************************************************************************/
__attribute__((optimize("-O0"))) Filter_Parm_Update(int Channel_To_Update)
{
int Temp_Ch1_Gain, Temp_Ch2_Gain;


//you need to use these to update the filter parameters
//int New_Filter_Vals;    /* use to flag new filter parms */
//int New_a0, New_a1, New_a2, New_b0, New_b1, New_b2;       /* Biquad filter coefficients to update */


/* Set up Temp variable for volumes...*/
Temp_Ch1_Gain = Data_Values.Volume_Gain_Sel * 5 + Data_Values.Ch_1_Att_Sel;
Temp_Ch2_Gain = Data_Values.Volume_Gain_Sel * 5 + Data_Values.Ch_2_Att_Sel;

/* for 4th order butterworth, need to cascade two filters	*/
/*					1st Q = 0.54		*/
/*					2nd Q = 1.31		*/
/* for 4th order LR, need to cascade two butterworth filters	*/
/*					1st Q = 0.707		*/
/*					2nd Q = 0.707		*/
/* for 2nd order butterworth, need one filter		*/
/*                                      1st Q = 1/SQRT(2)  = 0.707*/
/*                                      2nd passthrough*/
if (Channel_To_Update == Ch_1_Mode)
{
    if(Data_Values.Ch_1_SL_Sel == NoFilter)
    {
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_1_SL_Sel, Data_Values.Ch_1_FL_Sel, Q_BW, 0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 3;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_1_SL_Sel, Data_Values.Ch_1_FL_Sel, Q_BW, 0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 4;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_1_SL_Sel == SixdB)
    {
        Calc_DSP_Parms(F2_Type,  Data_Values.Ch_1_SL_Sel, Data_Values.Ch_1_FL_Sel, Q_BW, 0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 3;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_1_SL_Sel, Data_Values.Ch_1_FL_Sel, Q_BW, 0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 4;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_1_SL_Sel == TwelvedB)
    {
        Calc_DSP_Parms(F2_Type, Data_Values.Ch_1_SL_Sel, Data_Values.Ch_1_FL_Sel,  Q_BW,0 ,0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 3;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_1_SL_Sel, Data_Values.Ch_1_FL_Sel, Q_BW,0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 4;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_1_SL_Sel == TwentyFourdB)
    {
        Calc_DSP_Parms(F2_Type, TwelvedB, Data_Values.Ch_1_FL_Sel, Q_BW, 0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 3;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(F3_Type, TwelvedB, Data_Values.Ch_1_FL_Sel, Q_BW, 0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 4;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else;

    
    if(Data_Values.Ch_1_SH_Sel == NoFilter)
    {
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_1_SH_Sel, Data_Values.Ch_1_FH_Sel, Q_BW, Temp_Ch1_Gain , 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        /* implement channel inversion*/
        if (CH1_Invert == -1)
        {
            New_b0 = - New_b0;
            New_b1 = - New_b1;
            New_b2 = - New_b2;
        }
        else;
        New_Filter_Vals = 5;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_1_SH_Sel, Data_Values.Ch_1_FH_Sel, Q_BW, 0,              0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 6;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_1_SH_Sel == SixdB)
    {
        Calc_DSP_Parms(F4_Type, Data_Values.Ch_1_SH_Sel, Data_Values.Ch_1_FH_Sel,  Q_BW, Temp_Ch1_Gain, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        /* implement channel inversion*/
        if (CH1_Invert == -1)
        {
            New_b0 = - New_b0;
            New_b1 = - New_b1;
            New_b2 = - New_b2;
        }
        else;
        New_Filter_Vals = 5;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_1_SH_Sel, Data_Values.Ch_1_FH_Sel, Q_BW, 0,             0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 6;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_1_SH_Sel == TwelvedB)
    {
        Calc_DSP_Parms(F4_Type, Data_Values.Ch_1_SH_Sel, Data_Values.Ch_1_FH_Sel, Q_BW, Temp_Ch1_Gain, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        /* implement channel inversion*/
        if (CH1_Invert == -1)
        {
            New_b0 = - New_b0;
            New_b1 = - New_b1;
            New_b2 = - New_b2;
        }
        else;
        New_Filter_Vals = 5;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_1_SH_Sel, Data_Values.Ch_1_FH_Sel, Q_BW,            0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 6;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_1_SH_Sel == TwentyFourdB)
    {
        Calc_DSP_Parms(F4_Type, TwelvedB                , Data_Values.Ch_1_FH_Sel, Q_BW, Temp_Ch1_Gain, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        /* implement channel inversion*/
        if (CH1_Invert == -1)
        {
            New_b0 = - New_b0;
            New_b1 = - New_b1;
            New_b2 = - New_b2;
        }
        else;
        New_Filter_Vals = 5;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(F5_Type, TwelvedB                , Data_Values.Ch_1_FH_Sel, Q_BW,             0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 6;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
}
else if (Channel_To_Update == Ch_2_Mode)
{
    if(Data_Values.Ch_2_SL_Sel == NoFilter)
    {
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_2_SL_Sel, Data_Values.Ch_2_FL_Sel, Q_BW, Temp_Ch2_Gain, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        /* implement channel inversion*/
        if (CH2_Invert == -1)
        {
            New_b0 = - New_b0;
            New_b1 = - New_b1;
            New_b2 = - New_b2;
        }
        else;
        New_Filter_Vals = 7;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_2_SL_Sel, Data_Values.Ch_2_FL_Sel, Q_BW,             0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 8;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_2_SL_Sel == SixdB)
    {
        Calc_DSP_Parms(F6_Type, Data_Values.Ch_2_SL_Sel, Data_Values.Ch_2_FL_Sel, Q_BW, Temp_Ch2_Gain, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        /* implement channel inversion*/
        if (CH2_Invert == -1)
        {
            New_b0 = - New_b0;
            New_b1 = - New_b1;
            New_b2 = - New_b2;
        }
        else;
        New_Filter_Vals = 7;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_2_SL_Sel, Data_Values.Ch_2_FL_Sel, Q_BW,            0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 8;            /* set to the bank number plus 1*/
        do {   } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_2_SL_Sel == TwelvedB)
    {
        Calc_DSP_Parms(F6_Type, Data_Values.Ch_2_SL_Sel, Data_Values.Ch_2_FL_Sel, Q_BW, Temp_Ch2_Gain, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        /* implement channel inversion*/
        if (CH2_Invert == -1)
        {
            New_b0 = - New_b0;
            New_b1 = - New_b1;
            New_b2 = - New_b2;
        }
        else;
        New_Filter_Vals = 7;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(NoFilter, Data_Values.Ch_2_SL_Sel, Data_Values.Ch_2_FL_Sel, Q_BW,            0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 8;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.Ch_2_SL_Sel == TwentyFourdB)
    {
        Calc_DSP_Parms(F6_Type, TwelvedB, Data_Values.Ch_2_FL_Sel, Q_BW, Temp_Ch2_Gain, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        /* implement channel inversion*/
        if (CH2_Invert == -1)
        {
            New_b0 = - New_b0;
            New_b1 = - New_b1;
            New_b2 = - New_b2;
        }
        else;
        New_Filter_Vals = 7;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
        Calc_DSP_Parms(F7_Type, TwelvedB, Data_Values.Ch_2_FL_Sel, Q_BW,             0, 0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 8;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
}
else if (Channel_To_Update == Gen_Para_Mode)
{
    if(Data_Values.G1_Type_Sel == NoFilter)
    {
        Calc_DSP_Parms(NoFilter, Data_Values.G1_Type_Sel, Data_Values.G1_F_Sel,                                   0, 0,                                    0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 1;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.G1_Type_Sel == SixdB)
    {
        Calc_DSP_Parms(F0_LP_Type, Data_Values.G1_Type_Sel, Data_Values.G1_F_Sel,                              Q_BW, 0,                                    0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 1;            /* set to the bank number plus 1*/
        do {} while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.G1_Type_Sel == TwelvedB)
    {
        Calc_DSP_Parms(F0_LP_Type, Data_Values.G1_Type_Sel, Data_Values.G1_F_Sel,                              Q_BW, 0,                                    0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 1;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.G1_Type_Sel == TwentyFourdB)  /* just to catch errors - should never be this value as only have 1 biquad*/
    {
        Calc_DSP_Parms(F0_LP_Type, Data_Values.G1_Type_Sel, Data_Values.G1_F_Sel,                              Q_BW, 0,                                    0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 1;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.G1_Type_Sel == Peaking)
    {
        Calc_DSP_Parms(F0_Peaking, Data_Values.G1_Type_Sel, Data_Values.G1_F_Sel, (double)(Data_Values.G1_Q_Sel)/10, 0, (double)(Data_Values.G1_Gain_Sel)/10, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 1;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }

    if(Data_Values.G2_Type_Sel == NoFilter)
    {
        Calc_DSP_Parms(NoFilter, Data_Values.G2_Type_Sel, Data_Values.G2_F_Sel,                              Q_BW, 0,                                    0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 2;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.G2_Type_Sel == SixdB)
    {
        Calc_DSP_Parms(F0_LP_Type, Data_Values.G2_Type_Sel, Data_Values.G2_F_Sel,                              Q_BW, 0,                                    0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 2;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.G2_Type_Sel == TwelvedB)
    {
        Calc_DSP_Parms(F0_LP_Type, Data_Values.G2_Type_Sel, Data_Values.G2_F_Sel,                              Q_BW, 0,                                    0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 2;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.G2_Type_Sel == TwentyFourdB)   /* just to catch errors - should never be this value as only have 1 biquad*/
    {
        Calc_DSP_Parms(F0_LP_Type, Data_Values.G2_Type_Sel, Data_Values.G2_F_Sel,                              Q_BW, 0,                                    0, &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 2;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }
    else if(Data_Values.G2_Type_Sel == Peaking)
    {
        Calc_DSP_Parms(F0_Peaking, Data_Values.G2_Type_Sel, Data_Values.G2_F_Sel, (double)(Data_Values.G2_Q_Sel)/10, 0, (double)(Data_Values.G2_Gain_Sel)/10,  &New_a0, &New_a1, &New_a2, &New_b0, &New_b1, &New_b2);
        New_Filter_Vals = 2;            /* set to the bank number plus 1*/
        do { } while(New_Filter_Vals);     /* then wait till the ISR picks up the values, and clears the flag */
    }

}

}



/****************************************************************************/	
/* Uses the following globals                                               */
/* s_count                                                                  */
/* Old_Rot_Status    Records state of rotary control                        */
/* Fast_Loop_Counter  Used to set Fast_Cnt                                  */
/* Slow_Loop_Counter  Used to see if user turning control continuously      */
/*                                                                          */
/*                                                                          */
/* This sets is the ISR that deals with key presses etc                     */
/****************************************************************************/	
void __ISR(_TIMER_1_VECTOR, ipl4) Timer1ISR(void)
{
    unsigned char Current_Rot_Status = 0;

    /* this is debug - toggles an IO to show ISR operation*/
    //mPORTBToggleBits(BIT_9);
        /*Read Rot Status and mask off bits of interest */
	Current_Rot_Status = HMI_Port & (Vals_0 + Vals_1 + Exit + Sel);  
	Key_Just_Pressed = (Current_Rot_Status != Old_Rot_Status);

	/* Only do this if something has changed - but neither of the select or exit buttons are pressed */
	if (Key_Just_Pressed  && !(Current_Rot_Status & (Sel + Exit)))
	{
            /* Old Type Encoder is biphase input on two ports with */
            /*      Each detent click of the resolver being one change in phase*/
            /*      Thus one gray code change of the inputs occurs per click*/
            if Encoder_Type_Is_Old
            {
		if (Old_Rot_Status == Phase_0)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_1)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;
		}
		else if (Old_Rot_Status == Phase_1)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_2)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;			
		}
		else if (Old_Rot_Status == Phase_2)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_3)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;
		}
		else if (Old_Rot_Status == Phase_3)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_0)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;			
		}
            }
            /* Type A Encoder is biphase input on two ports with */
            /*      Each detent click of the resolver being four changes in phase*/
            /*      Thus one complete cycle of gray code occurs per click*/
            /*      Type A goes 11 -> 10 -> 00 -> 01 -> 11 and vice versa*/
            else if Encoder_Type_Is_TypeA
            {
                /* Phase_0 = 00*/
                /* From here need to work out if going up or down or reading crap*/
                /* CW 00 -> 01 */
                /* CCW 00-> 10 */
                /* But we do not want to act on these as it is mid-click*/
                if (Old_Rot_Status == Phase_0)
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
		}
                /* From here need to work out if going up or down or reading crap*/
                /* CW 01 -> 11 */
                /* CCW 01-> 00 */
                /* But we do not want to act on these as it is mid-click*/
		else if (Old_Rot_Status == Phase_1) 
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
		}
                /* From here need to work out if going up or down or reading crap*/
                /* CW 11 -> 10 */
                /* CCW 11-> 01 */
                /* This is it... do something*/
		else if (Old_Rot_Status == Phase_2)  
		{
			if (Current_Rot_Status == Phase_3)
				Keypress_Read = Rot_Down;
			else if (Current_Rot_Status == Phase_1)
				Keypress_Read = Rot_Up;
                        else Keypress_Read = 0;
		}
                /* From here need to work out if going up or down or reading crap*/
                /* CW 10 -> 00 */
                /* CCW 10-> 11 */
                /* But we do not want to act on these as it is mid-click*/
		else if (Old_Rot_Status == Phase_3) 
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
                }
            }
            /* Type B Encoder is biphase input on two ports with */
            /*      Each detent click of the resolver being three changes in phase*/
            /*      Thus one complete cycle of pseudo gray code occurs per click*/
            /*      Type B is asymetric for CW and CCW rotation*/
            /*      Type B goes 11 -> 00 -> 01 -> 11 for CW*/
            /*      Type B goes 00 -> 01 -> 00 -> 11 for CCW*/
            else if Encoder_Type_Is_TypeB
            {
                /* Phase_0 = 00*/
                /* From here need to work out if going up or down or reading crap*/
                /* CW 00 -> 01 */
                /* CCW 00-> 11 */
                /* But we do not want to act on these */
		if (Old_Rot_Status == Phase_0)
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
		}
                /* Phase_1 = 01*/
                /* From here need to work out if going up or down or reading crap*/
                /* CW  01 -> 11 */
                /* CCW 01 -> 00 */
                /* But we do not want to act on these */
		else if (Old_Rot_Status == Phase_1)
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
		}
                /* Phase_2 = 11*/
                /* OK This is it... */
                /* CW  11 -> 00 */
                /* CCW 11 -> 01 */
                /* But we do not want to act on these */
		else if (Old_Rot_Status == Phase_2)
		{
			if (Current_Rot_Status == Phase_0)
                            Keypress_Read = Rot_Up;
			else if (Current_Rot_Status == Phase_3)
                            Keypress_Read = Rot_Down;
                        else
                            Keypress_Read = 0;
		}
                /* Phase_3 = 10*/
                /* We are reading crap - illegal for this encoder*/
                /* We do not want to act on these */
		else if (Old_Rot_Status == Phase_3)  /* From here need to work out if going up or down */
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
                }
                else
                {
                    Keypress_Read = 0;
                }
            }
	
	Slow_Loop_Counter = Slow_Counter_Init;	/* If key just pressed initialise slow counter */
	
	if (Fast_Loop_Counter > (unsigned int)0)
		Fast_Loop_Counter--;
	}
	/* else if key pressed is not a rotation, then...*/
        else if (Key_Just_Pressed && (Current_Rot_Status & (Sel + Exit)))
	{
		if (Current_Rot_Status & Sel)
			Keypress_Read = Sel;			
		if (Current_Rot_Status & Exit)
			Keypress_Read = Exit;			
		Fast_Change = 1;
	}		
	else    /* else not Key_just_Pressed */
	{
		Keypress_Read = 0;
		if (Slow_Loop_Counter > (unsigned int)0)
			Slow_Loop_Counter--;
	}

	if (Slow_Loop_Counter == (unsigned int)0)
	{
		Fast_Change = 1;
		Fast_Loop_Counter = Fast_Counter_Init;
	}

	if (Fast_Loop_Counter == (unsigned int)0)   /* move into fast change of value for user */
	{
                Fast_Change = 100;
	}
			
	Old_Rot_Status = Current_Rot_Status;   /*Record Rot Status to allow comparison next ISR loop */

        /* This is debug */
//        mPORTBToggleBits(BIT_9);

        // set timer to zero then clear the interrupt flag
        TMR1 = 0x0000;
        mT1ClearIntFlag();
}

/****************************************************************************/
/* Uses the following globals                                               */
/*                                                                          */
/*                                                                          */
/*                                                                          */
// disable interrupts
//intStatus = INTDisableSystemMultiVectoredInt();
// do some critical code here
// restore interrupts to the previous state
//INTRestoreInterrupts(intStatus);
/*                                                                          */
/* This sets is the ISR that deals with the I2S interface                   */
/* It ultimately needs to generate DAC data at the sample rate              */

//int F0_a0, F0_a1, F0_a2, F0_b0, F0_b1, F0_b2;             /* Biquad filter coefficients 0 */
//int F1_a0, F1_a1, F1_a2, F1_b0, F1_b1, F1_b2;             /* Biquad filter coefficients 1 */
//int F2_a0, F2_a1, F2_a2, F2_b0, F2_b1, F2_b2;             /* Biquad filter coefficients 2 */
//int F3_a0, F3_a1, F3_a2, F3_b0, F3_b1, F3_b2;             /* Biquad filter coefficients 3 */
//int F4_a0, F4_a1, F4_a2, F4_b0, F4_b1, F4_b2;             /* Biquad filter coefficients 4 */
//int F5_a0, F5_a1, F5_a2, F5_b0, F5_b1, F5_b2;             /* Biquad filter coefficients 5 */
//int F6_a0, F6_a1, F6_a2, F6_b0, F6_b1, F6_b2;             /* Biquad filter coefficients 6 */
//int F7_a0, F7_a1, F7_a2, F7_b0, F7_b1, F7_b2;             /* Biquad filter coefficients 7 */

//int Xn_L, Xn_R;         /* input current sample*/
//int Xn1_L, Xn1_R;         /* Previous input current sample*/
//int Xn2_L, Xn2_R;         /* second prvious input current sample*/
//int Yn_L, Yn_R;         /* Output current sample*/
//int Yn1_L, Yn1_R;         /* Previous output current sample*/
//int Yn2_L, Yn2_R;         /* second previous output current sample*/

//Structure of filters is:
//  INPUT --> GEN1 --> GEN2 ----|---> HPF1 --> LPF1 --> Delay1 --> INV1 --> OUT1
//            F0       F1       |     F2, F3   F4, F5 
//                              |
//                              |---> HPF2 ------------ Delay2 --> INV2 --> OUT2
//                                    F6, F7

/****************************************************************************/
void __ISR(_SPI_2_VECTOR, ipl3SRS) SPI2InterruptHandler(void) {

    /* test Code */
//mPORTBToggleBits(BIT_8);
//Nop();



/* if filters have not changed then jump past the checks...*/
if (!New_Filter_Vals)
    {}
else
    {
    if(New_Filter_Vals == 1)
        {
            F0_a0 = New_a0;
            F0_a1 = New_a1;
            F0_a2 = New_a2;
            F0_b0 = New_b0;
            F0_b1 = New_b1;
            F0_b2 = New_b2;
            New_Filter_Vals = 0; /* clear the flag */
        }
    else if (New_Filter_Vals == 2)
        {
            F1_a0 = New_a0;
            F1_a1 = New_a1;
            F1_a2 = New_a2;
            F1_b0 = New_b0;
            F1_b1 = New_b1;
            F1_b2 = New_b2;
            New_Filter_Vals = 0; /* clear the flag */
        }
    else if (New_Filter_Vals == 3)
        {
            F2_a0 = New_a0;
            F2_a1 = New_a1;
            F2_a2 = New_a2;
            F2_b0 = New_b0;
            F2_b1 = New_b1;
            F2_b2 = New_b2;
            New_Filter_Vals = 0; /* clear the flag */
        }
    else if (New_Filter_Vals == 4)
        {
            F3_a0 = New_a0;
            F3_a1 = New_a1;
            F3_a2 = New_a2;
            F3_b0 = New_b0;
            F3_b1 = New_b1;
            F3_b2 = New_b2;
            New_Filter_Vals = 0; /* clear the flag */
        }
    else if (New_Filter_Vals == 5)
        {
            F4_a0 = New_a0;
            F4_a1 = New_a1;
            F4_a2 = New_a2;
            F4_b0 = New_b0;
            F4_b1 = New_b1;
            F4_b2 = New_b2;
            New_Filter_Vals = 0; /* clear the flag */
        }
    else if (New_Filter_Vals == 6)
        {
            F5_a0 = New_a0;
            F5_a1 = New_a1;
            F5_a2 = New_a2;
            F5_b0 = New_b0;
            F5_b1 = New_b1;
            F5_b2 = New_b2;
            New_Filter_Vals = 0; /* clear the flag */
        }
    else if (New_Filter_Vals == 7)
        {
            F6_a0 = New_a0;
            F6_a1 = New_a1;
            F6_a2 = New_a2;
            F6_b0 = New_b0;
            F6_b1 = New_b1;
            F6_b2 = New_b2;
            New_Filter_Vals = 0; /* clear the flag */
        }
     else if (New_Filter_Vals == 8)
        {
            F7_a0 = New_a0;
            F7_a1 = New_a1;
            F7_a2 = New_a2;
            F7_b0 = New_b0;
            F7_b1 = New_b1;
            F7_b2 = New_b2;
            New_Filter_Vals = 0; /* clear the flag */
        }
    else;
}

/* read the data */
Yn2_L = Yn1_L;          /* store the old value, you will need it!*/
Yn1_L = Yn_L;           /* store the old value, you will need it!*/
Xn2_L = Xn1_L;          /* store the old value, you will need it!*/
Xn1_L = Xn_L;           /* store the old value, you will need it!*/
Xn_L = SPI2BUF;
/* convert the 24 bit data into an integer */
if (Xn_L & 0x00800000)
    Xn_L = Xn_L | 0xff800000;
else;

Yn2_R = Yn1_R;          /* store the old value, you will need it!*/
Yn1_R = Yn_R;           /* store the old value, you will need it!*/
Xn2_R = Xn1_R;          /* store the old value, you will need it!*/
Xn1_R = Xn_R;           /* store the old value, you will need it!*/
Xn_R = SPI2BUF;
/* convert the 24 bit data into an integer */
if (Xn_R & 0x00800000)
    Xn_R = Xn_R | 0xff800000;
else;

/* implement filter 0  --  This is GENERAL Filter 1 G1*/
Acc_Reg =  (long long) F0_b0*Xn_L;
Acc_Reg += (long long) F0_b1*Xn1_L;
Acc_Reg += (long long) F0_b2*Xn2_L;
Acc_Reg += (long long) F0_a1*Yn1_L;
Acc_Reg += (long long) F0_a2*Yn2_L;
/* rescale */
Acc_Reg_Shift;
Yn_L_Assignment;

Acc_Reg =  (long long) F0_b0*Xn_R;
Acc_Reg += (long long) F0_b1*Xn1_R;
Acc_Reg += (long long) F0_b2*Xn2_R ;
Acc_Reg += (long long) F0_a1*Yn1_R ;
Acc_Reg += (long long) F0_a2*Yn2_R ;
/* rescale */
Acc_Reg_Shift;
Yn_R_Assignment;


/* copy and create history for data */
Y1n2_L = Y1n1_L;          /* store the old value, you will need it!*/
Y1n1_L = Y1n_L;           /* store the old value, you will need it!*/
X1n2_L = X1n1_L;          /* store the old value, you will need it!*/
X1n1_L = X1n_L;           /* store the old value, you will need it!*/
X1n_L = Yn_L;             /* this filter takes input from the output of F0 */

Y1n2_R = Y1n1_R;          /* store the old value, you will need it!*/
Y1n1_R = Y1n_R;           /* store the old value, you will need it!*/
X1n2_R = X1n1_R;          /* store the old value, you will need it!*/
X1n1_R = X1n_R;           /* store the old value, you will need it!*/
X1n_R = Yn_R;             /* this filter takes input from the output of F0 */

/* implement filter 1  --- THIS IS General Filter 2 G2*/
Acc_Reg = (long long) F1_b0*X1n_L;
Acc_Reg += (long long) F1_b1* X1n1_L;
Acc_Reg += (long long) F1_b2*X1n2_L;
Acc_Reg += (long long) F1_a1*Y1n1_L;
Acc_Reg += (long long) F1_a2*Y1n2_L;
/* rescale */
Acc_Reg_Shift;
Y1n_L_Assignment;

Acc_Reg = (long long) F1_b0*X1n_R;
Acc_Reg += (long long) F1_b1* X1n1_R;
Acc_Reg += (long long) F1_b2*X1n2_R ;
Acc_Reg += (long long) F1_a1*Y1n1_R ;
Acc_Reg += (long long) F1_a2*Y1n2_R ;
/* rescale */
Acc_Reg_Shift;
Y1n_R_Assignment;


/* copy and create history for data  --- This is HPF1, first Para*/
Y2n2_L = Y2n1_L;          /* store the old value, you will need it!*/
Y2n1_L = Y2n_L;           /* store the old value, you will need it!*/
X2n2_L = X2n1_L;          /* store the old value, you will need it!*/
X2n1_L = X2n_L;           /* store the old value, you will need it!*/
X2n_L =  Y1n_L;           /* this filter takes input from the output of F1 */

Y2n2_R = Y2n1_R;          /* store the old value, you will need it!*/
Y2n1_R = Y2n_R;           /* store the old value, you will need it!*/
X2n2_R = X2n1_R;          /* store the old value, you will need it!*/
X2n1_R = X2n_R;           /* store the old value, you will need it!*/
X2n_R =  Y1n_R;           /* this filter takes input from the output of F1 */

/* implement filter 1 */
Acc_Reg = (long long) F2_b0*X2n_L;
Acc_Reg += (long long) F2_b1* X2n1_L;
Acc_Reg += (long long) F2_b2*X2n2_L;
Acc_Reg += (long long) F2_a1*Y2n1_L;
Acc_Reg += (long long) F2_a2*Y2n2_L;
/* rescale */
Acc_Reg_Shift;
Y2n_L_Assignment;

Acc_Reg = (long long) F2_b0*X2n_R;
Acc_Reg += (long long) F2_b1* X2n1_R;
Acc_Reg += (long long) F2_b2*X2n2_R ;
Acc_Reg += (long long) F2_a1*Y2n1_R ;
Acc_Reg += (long long) F2_a2*Y2n2_R ;
/* rescale */
Acc_Reg_Shift;
Y2n_R_Assignment;

/* coy and create history for data --- This is HPF1, Second Para */
Y3n2_L = Y3n1_L;          /* store the old value, you will need it!*/
Y3n1_L = Y3n_L;           /* store the old value, you will need it!*/
X3n2_L = X3n1_L;          /* store the old value, you will need it!*/
X3n1_L = X3n_L;           /* store the old value, you will need it!*/
X3n_L =  Y2n_L;           /* this filter takes input from the output of F2 */

Y3n2_R = Y3n1_R;          /* store the old value, you will need it!*/
Y3n1_R = Y3n_R;           /* store the old value, you will need it!*/
X3n2_R = X3n1_R;          /* store the old value, you will need it!*/
X3n1_R = X3n_R;           /* store the old value, you will need it!*/
X3n_R =  Y2n_R;           /* this filter takes input from the output of F2 */

/* implement filter 1 */
Acc_Reg = (long long) F3_b0*X3n_L;
Acc_Reg += (long long) F3_b1* X3n1_L;
Acc_Reg += (long long) F3_b2*X3n2_L;
Acc_Reg += (long long) F3_a1*Y3n1_L;
Acc_Reg += (long long) F3_a2*Y3n2_L;
/* rescale */
Acc_Reg_Shift;
Y3n_L_Assignment;

Acc_Reg = (long long) F3_b0*X3n_R;
Acc_Reg += (long long) F3_b1* X3n1_R;
Acc_Reg += (long long) F3_b2*X3n2_R ;
Acc_Reg += (long long) F3_a1*Y3n1_R ;
Acc_Reg += (long long) F3_a2*Y3n2_R ;
/* rescale */
Acc_Reg_Shift;
Y3n_R_Assignment;

/* copy and create history for data --- This is LPF1, first Para*/
Y4n2_L = Y4n1_L;          /* store the old value, you will need it!*/
Y4n1_L = Y4n_L;           /* store the old value, you will need it!*/
X4n2_L = X4n1_L;          /* store the old value, you will need it!*/
X4n1_L = X4n_L;           /* store the old value, you will need it!*/
X4n_L =  Y3n_L;           /* this filter takes input from the output of F3 */

Y4n2_R = Y4n1_R;          /* store the old value, you will need it!*/
Y4n1_R = Y4n_R;           /* store the old value, you will need it!*/
X4n2_R = X4n1_R;          /* store the old value, you will need it!*/
X4n1_R = X4n_R;           /* store the old value, you will need it!*/
X4n_R =  Y3n_R;           /* this filter takes input from the output of F3 */

/* implement filter 1 */
Acc_Reg = (long long) F4_b0*X4n_L;
Acc_Reg += (long long) F4_b1* X4n1_L;
Acc_Reg += (long long) F4_b2*X4n2_L;
Acc_Reg += (long long) F4_a1*Y4n1_L;
Acc_Reg += (long long) F4_a2*Y4n2_L;
/* rescale */
Acc_Reg_Shift;
Y4n_L_Assignment;

Acc_Reg = (long long) F4_b0*X4n_R;
Acc_Reg += (long long) F4_b1* X4n1_R;
Acc_Reg += (long long) F4_b2*X4n2_R ;
Acc_Reg += (long long) F4_a1*Y4n1_R ;
Acc_Reg += (long long) F4_a2*Y4n2_R ;
/* rescale */
Acc_Reg_Shift;
Y4n_R_Assignment;

/* copy and create history for data --- This is LPF1, Second Para */
Y5n2_L = Y5n1_L;          /* store the old value, you will need it!*/
Y5n1_L = Y5n_L;           /* store the old value, you will need it!*/
X5n2_L = X5n1_L;          /* store the old value, you will need it!*/
X5n1_L = X5n_L;           /* store the old value, you will need it!*/
X5n_L =  Y4n_L;           /* this filter takes input from the output of F4 */

Y5n2_R = Y5n1_R;          /* store the old value, you will need it!*/
Y5n1_R = Y5n_R;           /* store the old value, you will need it!*/
X5n2_R = X5n1_R;          /* store the old value, you will need it!*/
X5n1_R = X5n_R;           /* store the old value, you will need it!*/
X5n_R =  Y4n_R;           /* this filter takes input from the output of F4 */

/* implement filter 1 */
Acc_Reg = (long long)  F5_b0*X5n_L;
Acc_Reg += (long long) F5_b1*X5n1_L;
Acc_Reg += (long long) F5_b2*X5n2_L;
Acc_Reg += (long long) F5_a1*Y5n1_L;
Acc_Reg += (long long) F5_a2*Y5n2_L;
/* rescale */
Acc_Reg_Shift;
Y5n_L_Assignment;

Acc_Reg  = (long long) F5_b0*X5n_R;
Acc_Reg += (long long) F5_b1*X5n1_R;
Acc_Reg += (long long) F5_b2*X5n2_R ;
Acc_Reg += (long long) F5_a1*Y5n1_R ;
Acc_Reg += (long long) F5_a2*Y5n2_R ;
/* rescale */
Acc_Reg_Shift;
Y5n_R_Assignment;

/* copy and create history for data --- This is HPF2, first Para*/
Y6n2_L = Y6n1_L;          /* store the old value, you will need it!*/
Y6n1_L = Y6n_L;           /* store the old value, you will need it!*/
X6n2_L = X6n1_L;          /* store the old value, you will need it!*/
X6n1_L = X6n_L;           /* store the old value, you will need it!*/
X6n_L =  Y1n_L;           /* this filter takes input from the output of F1 */
                          /* in output of GEN2 para */

Y6n2_R = Y6n1_R;          /* store the old value, you will need it!*/
Y6n1_R = Y6n_R;           /* store the old value, you will need it!*/
X6n2_R = X6n1_R;          /* store the old value, you will need it!*/
X6n1_R = X6n_R;           /* store the old value, you will need it!*/
X6n_R =  Y1n_R;           /* this filter takes input from the output of F1 */
                          /* in output of GEN2 para */
/* implement filter 1 */
Acc_Reg = (long long)  F6_b0*X6n_L;
Acc_Reg += (long long) F6_b1*X6n1_L;
Acc_Reg += (long long) F6_b2*X6n2_L;
Acc_Reg += (long long) F6_a1*Y6n1_L;
Acc_Reg += (long long) F6_a2*Y6n2_L;
/* rescale */
Acc_Reg_Shift;
Y6n_L_Assignment;

Acc_Reg  = (long long) F6_b0*X6n_R;
Acc_Reg += (long long) F6_b1*X6n1_R;
Acc_Reg += (long long) F6_b2*X6n2_R;
Acc_Reg += (long long) F6_a1*Y6n1_R;
Acc_Reg += (long long) F6_a2*Y6n2_R;
/* rescale */
Acc_Reg_Shift;
Y6n_R_Assignment;

/* copy and create history for data  --- This is HPF2, second Para*/
Y7n2_L = Y7n1_L;          /* store the old value, you will need it!*/
Y7n1_L = Y7n_L;           /* store the old value, you will need it!*/
X7n2_L = X7n1_L;          /* store the old value, you will need it!*/
X7n1_L = X7n_L;           /* store the old value, you will need it!*/
X7n_L =  Y6n_L;           /* this filter takes input from the output of F6 */

Y7n2_R = Y7n1_R;          /* store the old value, you will need it!*/
Y7n1_R = Y7n_R;           /* store the old value, you will need it!*/
X7n2_R = X7n1_R;          /* store the old value, you will need it!*/
X7n1_R = X7n_R;           /* store the old value, you will need it!*/
X7n_R =  Y6n_R;           /* this filter takes input from the output of F6 */

/* implement filter 1 */
Acc_Reg = (long long)  F7_b0*X7n_L;
Acc_Reg += (long long) F7_b1*X7n1_L;
Acc_Reg += (long long) F7_b2*X7n2_L;
Acc_Reg += (long long) F7_a1*Y7n1_L;
Acc_Reg += (long long) F7_a2*Y7n2_L;
/* rescale */
Acc_Reg_Shift;
Y7n_L_Assignment;

Acc_Reg  = (long long) F7_b0*X7n_R;
Acc_Reg += (long long) F7_b1*X7n1_R;
Acc_Reg += (long long) F7_b2*X7n2_R;
Acc_Reg += (long long) F7_a1*Y7n1_R;
Acc_Reg += (long long) F7_a2*Y7n2_R;
/* rescale */
Acc_Reg_Shift;
Y7n_R_Assignment;

/* copy internal filter data to output variables - allows us to apply limits*/
/* while allowing the filters to operate internally with much larger values */
/* than the DAC can output*/
CH1_Output_Data_L = Y5n_L;
CH1_Output_Data_R = Y5n_R;
CH2_Output_Data_L = Y7n_L;
CH2_Output_Data_R = Y7n_R;


Out_Test_Temp = Y5n_L & DAC_Valid_BitMask;
if (Out_Test_Temp & 0x80000000)
    {
        if (!(Out_Test_Temp ^ 0xff800000))
            CH1_Output_Data_L = Y5n_L;
        else
            CH1_Output_Data_L = -DAC_Output_Max;
    }
else if (Out_Test_Temp)
    CH1_Output_Data_L = DAC_Output_Max;
else
    CH1_Output_Data_L = Y5n_L;


Out_Test_Temp = Y5n_R & DAC_Valid_BitMask;
if (Out_Test_Temp & 0x80000000)
    {
        if (!(Out_Test_Temp ^ 0xff800000))
            CH1_Output_Data_R = Y5n_R;
        else
            CH1_Output_Data_R = -DAC_Output_Max;
    }
else if (Out_Test_Temp)
    CH1_Output_Data_R = DAC_Output_Max;
else
    CH1_Output_Data_R = Y5n_R;

/* now channel 2 */
Out_Test_Temp = Y7n_L & DAC_Valid_BitMask;
if (Out_Test_Temp & 0x80000000)
    {
        if (!(Out_Test_Temp ^ 0xff800000))
            CH2_Output_Data_L = Y7n_L;
        else
            CH2_Output_Data_L = -DAC_Output_Max;
    }
else if (Out_Test_Temp)
    CH2_Output_Data_L = DAC_Output_Max;
else
    CH2_Output_Data_L = Y7n_L;


Out_Test_Temp = Y7n_R & DAC_Valid_BitMask;
if (Out_Test_Temp & 0x80000000)
    {
        if (!(Out_Test_Temp ^ 0xff800000))
            CH2_Output_Data_R = Y7n_R;
        else
            CH2_Output_Data_R = -DAC_Output_Max;
    }
else if (Out_Test_Temp)
    CH2_Output_Data_R = DAC_Output_Max;
else
    CH2_Output_Data_R = Y7n_R;

/* cheat here and do not mask off the top bits - as they have been cleared above */
*(Delay_Buffer_Ch1_L + Write_Pointer_Offset) = CH1_Output_Data_L;
*(Delay_Buffer_Ch1_R + Write_Pointer_Offset) = CH1_Output_Data_R;
/* point the write pointers at the right spot*/
*(Delay_Buffer_Ch2_L + Write_Pointer_Offset) = CH2_Output_Data_L;
*(Delay_Buffer_Ch2_R + Write_Pointer_Offset) = CH2_Output_Data_R;

Write_Pointer_Offset++;
Write_Pointer_Offset = Write_Pointer_Offset & Delay_Buf_Size_Mask;

SPI2BUF = *(Delay_Buffer_Ch1_L + Ch1_Rd_Offset);
SPI2BUF = *(Delay_Buffer_Ch1_R + Ch1_Rd_Offset);

SPI1BUF = *(Delay_Buffer_Ch2_L + Ch2_Rd_Offset);
SPI1BUF = *(Delay_Buffer_Ch2_R + Ch2_Rd_Offset);

/* increment read pointers */
Ch1_Rd_Offset++;
Ch2_Rd_Offset++;
Ch1_Rd_Offset = Ch1_Rd_Offset & Delay_Buf_Size_Mask;
Ch2_Rd_Offset = Ch2_Rd_Offset & Delay_Buf_Size_Mask;



//mPORTBToggleBits(BIT_8);
//Nop();



INTClearFlag(INT_SPI2);

}

/****************************************************************************/
/*                                                                          */
/* The main DDS function is in an ISR                                       */
/* The main function simply implements the HMI and sets up parameters for   */
/* the DDS.  The following variables are stored globally to ensure they are */
/* accessible to the ISR without going through too much hassle              */
/*                                                                          */
/* DDS data storage                                                         */
/* Phase_Acc_Ch1, Phase_Acc_Ch2;    phase accumulators                      */
/* Phase_Inc_Ch1, Phase_Inc_Ch2     phase increment per clock               */
/* Cycles_On_Ch1, Cycles_On_Ch2;    number of cycles on for pulse mode      */
/* Cycles_Off_Ch1, Cycles_Off_Ch2;  number of cycles off for pulse mode     */
/* FM_Phase_Acc_Ch1, FM_Phase_Acc_Ch2; phase accumulator for FM             */
/* AM_Phase_Acc_Ch1, AM_Phase_Acc_Ch2; phase accumulator for AM             */
/****************************************************************************/
void main(void) {
char Update_Display = 1; 		/* Flag saying need to update display */
int Freq_Speed, Freq_Speed_Counter = 1; /* use this to keep track of How Fast to count Frequency */
long int Auto_Revert = Auto_Revert_Init_Val;	
					/* Use this as a counter - after 255 loops auto revert to volume mode */
char Band = 0;           		/* use to drive state machine band select */
int Step_Size = 0;        		/* Use this to implement push button steps */  
int Display_Value = 0;                  /* use this to display the current value */
int Mode = Ch_1_Mode;                   /* use this to track current mode */
char Memory_Bank = Default_Mem_Bank;    /* use this to track which memory bank is in use */
unsigned int Channel_Mem_Offset;        /* used to implement channel 1 & 2 offset on save call */
int Sel_Fn = Choosing_Fn;		/* Use this to track current Selection */
int spi_con1, spi_con2;                 /* local storage */
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//STEP 1. Configure cache, wait states and peripheral bus clock
// Configure the device for maximum performance but do not change the PBDIV
// Given the options, this function will change the flash wait states, RAM
// wait state and enable prefetch cache but will not change the PBDIV.
// The PBDIV value is already set via the pragma FPBDIV option above..
SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

/* set up output data pointer*/
Out_Data = &Acc_Reg; /* point Out_Data at bottom word of Acc_Reg*/


// configure for multi-vectored mode
INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);

/* intitialise the Micro I/O ports*/
Init_IO();
/* start with input muted */
Input_Mute = 0;
/*Allows LCD to settle*/
DelayMs(100);
CselClearEEPROM;                        /* just to be clear on this for the EEPROM*/
Init_LCD();				/* initialise the LCD */
Update_LCD();				/* Writes Line1 and Line2 to LCD */
DelayMs(500);
DelayMs(500);
DelayMs(500);
/* Initialise and load parameters from EEPROM - we need these to load defaults & saved values */

Load_From_Memory(Default_Mem_Bank); /* Read Data from ROM */

/****************************************************************************/	
/* Get reasonable initialisation, go straight to volume			*/
/****************************************************************************/	
Sel_Fn = Volume_Sel_Fn;
Display_Value = Data_Values.Volume_Gain_Sel;
Display_Data(Sel_Fn, Display_Value);

/* set invert variables*/
CH1_Invert = 1;
CH2_Invert = 1;

/* initialise delays*/
CH1_Delay_Offset = Data_Values.Ch_1_Del_Sel * SpeedOfSound;
CH1_Delay_Offset = CH1_Delay_Offset / Sample_Rate;
CH2_Delay_Offset = Data_Values.Ch_2_Del_Sel * SpeedOfSound;
CH2_Delay_Offset = CH1_Delay_Offset / Sample_Rate;

/* initialise the pointers */
Write_Pointer_Offset = 0;  /*start things with sero offset*/
Ch1_Rd_Offset = (Write_Pointer_Offset - Data_Values.Ch_1_Del_Sel) & Delay_Buf_Size_Mask ;
Ch2_Rd_Offset = (Write_Pointer_Offset - Data_Values.Ch_2_Del_Sel) & Delay_Buf_Size_Mask ;

New_Filter_Vals = 0;    /* use to flag no new filter parms - set o dilter set number to update*/

/* give things a bit to settle */
Step_Size = 0;
Key_Just_Pressed = 0;


/****************************************************************************/
/* This sets up the test ISR 					  	    */
/* Should be about 500us cycle 						    */
/****************************************************************************/
/* We will eventually want to be able to set the following: */
/* IEC0 bit 4, Timer1 */
/* IEC0 bit 9, Timer2 */
/* IEC1 bit 2, USB */
/* IEC1 bit 5, SPI1TX */
//turn t1 off
T1CON = 0X0;
// Clear the timer
TMR1 = 0x0000;
//Sets period register
PR1 = 0xFFFF;
// Open the timer
// T1_TICK = 1/256 seconds - we want 500uS
// T1_TICK/4 is what we want...
OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_8, (T1_TICK/4));
// set up the timer interrupt with a priority of 2
//INTSetVectorPriority(INT_TIMER_1_VECTOR, INT_PRIORITY_LEVEL_4);
//INTSetVectorSubPriority(INT_TIMER_1_VECTOR, INT_SUB_PRIORITY_LEVEL_3);
ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_2 | T1_INT_SUB_PRIOR_3);
INTClearFlag(INT_T1);
INTEnable(INT_T1, 1);

//Enable SPI1 interrupt.
mSPI2IntDisable();
INTClearFlag(INT_SPI2);
ConfigIntSPI2(SPI_FAULT_INT_DIS | SPI_TX_INT_EN | SPI_RX_INT_DIS | SPI_INT_SUB_PRI_2 | SPI_INT_PRI_3);
SpiChnClrIntFlags(2);
INTEnableInterrupts();

/* turn on channel 1*/
spi_con1 = SPI_OPEN_MSTEN | //Master mode enable
        SPI_OPEN_SSEN | //Enable slave select function
        SPI_OPEN_CKP_HIGH | //Clock polarity Idle High Actie Low
        SPI_OPEN_ENHBUF | // Enhanced Mode
        SPI_OPEN_TBE_HALF_EMPTY | //generate event when TX buffer more than half empty
        SPI_OPEN_MODE16 | //Data mode: 24b
        SPI_OPEN_MODE32 | //Data mode: 24b
        SPI_OPEN_MCLKSEL | //Clock selected is reference clock
        SPI_OPEN_FSP_HIGH; //Frame Sync Pulse is active high

spi_con2 = SPI_OPEN2_AUDEN | //Enable Audio mode
        SPI_OPEN2_AUDMOD_I2S; //Enable I2S mode

SPI2CON2 = SPI2CON2 & 0xfffffdff; // ignore receive overflow
SPI2STATCLR = 1<<6; // clear receive overflow
SPI2CON = SPI2CON | 0x00008000; // turn SPI2 back on
SPI2BUF = 0x00000000;
SPI2BUF = 0x00000000;

//temp deletion##################################################################
SpiChnOpenEx(SPI_CHANNEL1, spi_con1, spi_con2, (mclk / baud_rate));
SPI2CON = SPI1CON | 0x00008000; // turn SPI2 back on
//SPI2BUF = 0x00000000;
//SPI2BUF = 0x00000000;

/* Update Filters*/
Filter_Parm_Update(Gen_Para_Mode);
Filter_Parm_Update(Ch_1_Mode);
Filter_Parm_Update(Ch_2_Mode);
/* and then unmute the input */
Input_Mute = 1;

//GLCD_Rectangle(1,1,20,20);
//do {} while (1);

/* the main loop that runs the HMI - when it is not being interrupted!*/
do
{
/* Look after changing modes */

    mPORTBToggleBits(BIT_10);
    Nop();

	if (Key_Just_Pressed)
	{
		Update_Display = 1;
		Auto_Revert = Auto_Revert_Init_Val;		/* If key pressed, reset the counter */

		/********************************************************************/
		/* Currently in Change Mode - act on legal button presses 			*/
		/********************************************************************/
		if (Sel_Fn == Choosing_Fn)
		{
			/* Up or Down*/
			if (Up_Keys || Down_Keys)
			{
				if (Up_Keys)
					Mode += 1;
				else
					Mode -= 1;
				/* check for wrap around */
				if (Mode > Load_Mode)
					Mode = Ch_1_Mode;
				if (Mode < Ch_1_Mode)
					Mode = Load_Mode;
			}

			/* Select Key hit */
			else if Sel_Key
			{
				Sel_Fn = Mode*10;
			}
			/* Exit */
			else if Exit_Key
			{
				Sel_Fn = Choosing_Fn;
			}
			else;
			
			if (Sel_Fn == Choosing_Fn)
				Display_Value = Mode;
                        else if (Sel_Fn == Ch_1_FL)
				Display_Value = Data_Values.Ch_1_FL_Sel;
                        else if (Sel_Fn == Ch_2_FL)
				Display_Value = Data_Values.Ch_2_FL_Sel;
                        else if (Sel_Fn == G1_Type)
				Display_Value = Data_Values.G1_Type_Sel;
                        else if (Sel_Fn == Volume_Sel_Fn)
				Display_Value = Data_Values.Volume_Gain_Sel;
                        else if (Sel_Fn == Load_Sel_Fn)
				Display_Value = Memory_Bank;
                        else if (Sel_Fn == Save_Sel_Fn)
				Display_Value = Memory_Bank;
			else;
                }

		/********************************************************************/
		/* Currently in Ch_1_FL Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Ch_1_FL)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/
   
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_FL_Sel = Data_Values.Ch_1_FL_Sel - Freq_Speed;
				if ((Data_Values.Ch_1_FL_Sel) <  Freq_Min)
                                        Data_Values.Ch_1_FL_Sel =  Freq_Min;
				Display_Value = Data_Values.Ch_1_FL_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_FL_Sel = Data_Values.Ch_1_FL_Sel + Freq_Speed;
				if ((Data_Values.Ch_1_FL_Sel) >  Freq_Max)
                                        Data_Values.Ch_1_FL_Sel =  Freq_Max;
				Display_Value = Data_Values.Ch_1_FL_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Freq_Speed_Counter_1_fn;
				Display_Value = Freq_Speed_Counter;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Ch_1_Mode);
                }

      		/********************************************************************/
		/* Currently in Ch_2_FL Function - act on legal button presses    */
		/********************************************************************/
                else if (Sel_Fn == Ch_2_FL)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_2_FL_Sel = Data_Values.Ch_2_FL_Sel - Freq_Speed;
				if ((Data_Values.Ch_2_FL_Sel) <  Freq_Min)
                                        Data_Values.Ch_2_FL_Sel =  Freq_Min;
				Display_Value = Data_Values.Ch_2_FL_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_2_FL_Sel = Data_Values.Ch_2_FL_Sel + Freq_Speed;
				if ((Data_Values.Ch_2_FL_Sel) >  Freq_Max)
                                        Data_Values.Ch_2_FL_Sel =  Freq_Max;
				Display_Value = Data_Values.Ch_2_FL_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Freq_Speed_Counter_2_fn;
				Display_Value = Freq_Speed_Counter;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Ch_2_Mode);
                }

      		/********************************************************************/
		/* Currently in Freq_Speed_Counter Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Freq_Speed_Counter_1_fn)
		{
			/* Down */
			if Down_Keys
			{
				Freq_Speed_Counter = Freq_Speed_Counter/10;
				if ((Freq_Speed_Counter) <  Min_Freq_Steps)
                                        Freq_Speed_Counter =  Min_Freq_Steps;
				Display_Value = Freq_Speed_Counter;
			}
			/* Up */
			else if Up_Keys
			{
				Freq_Speed_Counter = Freq_Speed_Counter * 10;
				if ((Freq_Speed_Counter) >  Max_Freq_Steps)
                                        Freq_Speed_Counter =  Max_Freq_Steps;
				Display_Value = Freq_Speed_Counter;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_1_SL;
				Display_Value = Data_Values.Ch_1_SL_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Ch_1_FL;
				Display_Value = Data_Values.Ch_1_FL_Sel;
                        }
			else;
		}

       		/********************************************************************/
		/* Currently in Freq_Speed_Counter Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Freq_Speed_Counter_2_fn)
		{
			/* Down */
			if Down_Keys
			{
				Freq_Speed_Counter = Freq_Speed_Counter/10;
				if ((Freq_Speed_Counter) <  Min_Freq_Steps)
                                        Freq_Speed_Counter =  Min_Freq_Steps;
				Display_Value = Freq_Speed_Counter;
			}
			/* Up */
			else if Up_Keys
			{
				Freq_Speed_Counter = Freq_Speed_Counter * 10;
				if ((Freq_Speed_Counter) >  Max_Freq_Steps)
                                        Freq_Speed_Counter =  Max_Freq_Steps;
				Display_Value = Freq_Speed_Counter;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_2_SL;
				Display_Value = Data_Values.Ch_2_SL_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Ch_2_FL;
				Display_Value = Data_Values.Ch_2_FL_Sel;
                        }
			else;
		}

                /***********************************************************************/
		/* Currently in Ch_1_SL Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == Ch_1_SL)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_SL_Sel = Data_Values.Ch_1_SL_Sel - 1;
				if ((Data_Values.Ch_1_SL_Sel) <  Filter_Min)
                                        Data_Values.Ch_1_SL_Sel =  Filter_Max;
				Display_Value = Data_Values.Ch_1_SL_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_SL_Sel = Data_Values.Ch_1_SL_Sel + 1;
				if ((Data_Values.Ch_1_SL_Sel) > Filter_Max)
                                    Data_Values.Ch_1_SL_Sel = Filter_Min;
    				Display_Value = Data_Values.Ch_1_SL_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Filter_Parm_Update(Ch_1_Mode);
                            Sel_Fn = Ch_1_FH;
                            Display_Value = Data_Values.Ch_1_FH_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Ch_1_Mode);
               }
                /***********************************************************************/
		/* Currently in Ch_2_SL Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == Ch_2_SL)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_2_SL_Sel = Data_Values.Ch_2_SL_Sel - 1;
				if ((Data_Values.Ch_2_SL_Sel) <  Filter_Min)
                                        Data_Values.Ch_2_SL_Sel =  Filter_Max;
				Display_Value = Data_Values.Ch_2_SL_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_2_SL_Sel = Data_Values.Ch_2_SL_Sel + 1;
				if ((Data_Values.Ch_2_SL_Sel) > Filter_Max)
                                    Data_Values.Ch_2_SL_Sel = Filter_Min;
    				Display_Value = Data_Values.Ch_2_SL_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Filter_Parm_Update(Ch_2_Mode);
                            Sel_Fn = Ch_2_Del;
                            Display_Value = Data_Values.Ch_2_Del_Sel * SpeedOfSound;
                            Display_Value = Display_Value / Sample_Rate;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Ch_2_Mode);
                }
		/********************************************************************/
		/* Currently in Ch_1_FH Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Ch_1_FH)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_FH_Sel = Data_Values.Ch_1_FH_Sel - Freq_Speed;
				if ((Data_Values.Ch_1_FH_Sel) <  Freq_Min)
                                        Data_Values.Ch_1_FH_Sel =  Freq_Min;
				Display_Value = Data_Values.Ch_1_FH_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_FH_Sel = Data_Values.Ch_1_FH_Sel + Freq_Speed;
				if ((Data_Values.Ch_1_FH_Sel) >  Freq_Max)
                                        Data_Values.Ch_1_FH_Sel =  Freq_Max;
				Display_Value = Data_Values.Ch_1_FH_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_1_SH;
				Display_Value = Data_Values.Ch_1_SH_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Ch_1_Mode);
                }

                /***********************************************************************/
		/* Currently in Ch_1_SH Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == Ch_1_SH)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_SH_Sel = Data_Values.Ch_1_SH_Sel - 1;
				if ((Data_Values.Ch_1_SH_Sel) <  Filter_Min)
                                        Data_Values.Ch_1_SH_Sel =  Filter_Max;
				Display_Value = Data_Values.Ch_1_SH_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_SH_Sel = Data_Values.Ch_1_SH_Sel + 1;
				if ((Data_Values.Ch_1_SH_Sel) > Filter_Max)
                                    Data_Values.Ch_1_SH_Sel = Filter_Min;
    				Display_Value = Data_Values.Ch_1_SH_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Filter_Parm_Update(Ch_1_Mode);
                            Sel_Fn = Ch_1_Del;
                            Display_Value = Data_Values.Ch_1_Del_Sel * SpeedOfSound;
                            Display_Value = Display_Value / Sample_Rate;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Ch_1_Mode);
               }
		/********************************************************************/
		/* Currently in Ch_1_Del Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Ch_1_Del)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_Del_Sel = Data_Values.Ch_1_Del_Sel - Freq_Speed;
				if ((Data_Values.Ch_1_Del_Sel) <  Del_Min)
                                        Data_Values.Ch_1_Del_Sel =  Del_Min;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_Del_Sel = Data_Values.Ch_1_Del_Sel + Freq_Speed;
				if ((Data_Values.Ch_1_Del_Sel) >  Del_Max)
                                        Data_Values.Ch_1_Del_Sel =  Del_Max;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_1_Inv;
				Display_Value = Data_Values.Ch_1_Inv_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update delay offset value */
                        Display_Value = Data_Values.Ch_1_Del_Sel * SpeedOfSound;
                        Display_Value = Display_Value / Sample_Rate;
                        Ch1_Rd_Offset = (Write_Pointer_Offset - Data_Values.Ch_1_Del_Sel) & Delay_Buf_Size_Mask ;
                }
		/********************************************************************/
		/* Currently in Ch_2_Del Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Ch_2_Del)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_2_Del_Sel = Data_Values.Ch_2_Del_Sel - Freq_Speed;
				if ((Data_Values.Ch_2_Del_Sel) <  Del_Min)
                                        Data_Values.Ch_2_Del_Sel =  Del_Min;
				Display_Value = Data_Values.Ch_2_Del_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_2_Del_Sel = Data_Values.Ch_2_Del_Sel + Freq_Speed;
				if ((Data_Values.Ch_2_Del_Sel) >  Del_Max)
                                        Data_Values.Ch_2_Del_Sel =  Del_Max;
				Display_Value = Data_Values.Ch_2_Del_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_2_Inv;
				Display_Value = Data_Values.Ch_2_Inv_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update delay offset value */
                        Display_Value = Data_Values.Ch_2_Del_Sel * SpeedOfSound;
                        Display_Value = Display_Value / Sample_Rate;
                        Ch2_Rd_Offset = (Write_Pointer_Offset - Data_Values.Ch_2_Del_Sel) & Delay_Buf_Size_Mask ;
                }
                /********************************************************************/
		/* Currently in Ch_1_Inv Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Ch_1_Inv)
		{
			/* Down */
			if Down_Keys
			{
                            if ((Data_Values.Ch_1_Inv_Sel) ==  CHNonInverted )
                                    Data_Values.Ch_1_Inv_Sel =  CHInverted;
                            else
                                    Data_Values.Ch_1_Inv_Sel =  CHNonInverted;
                            Display_Value = Data_Values.Ch_1_Inv_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            if ((Data_Values.Ch_1_Inv_Sel) ==  CHNonInverted )
                                    Data_Values.Ch_1_Inv_Sel =  CHInverted;
                            else
                                    Data_Values.Ch_1_Inv_Sel =  CHNonInverted;
                            Display_Value = Data_Values.Ch_1_Inv_Sel;
			}
			else if Sel_Key
			{
                            Sel_Fn = Ch_1_Att;
                            Display_Value = Data_Values.Ch_1_Att_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* set up invert variables */
                        CH1_Invert = Data_Values.Ch_1_Inv_Sel;
                        Filter_Parm_Update(Ch_1_Mode);
		}
                /********************************************************************/
		/* Currently in Ch_2_Inv Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Ch_2_Inv)
		{
			/* Down */
			if Down_Keys
			{
                            if ((Data_Values.Ch_2_Inv_Sel) ==  CHNonInverted )
                                    Data_Values.Ch_2_Inv_Sel =  CHInverted;
                            else
                                    Data_Values.Ch_2_Inv_Sel =  CHNonInverted;
                            Display_Value = Data_Values.Ch_2_Inv_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            if ((Data_Values.Ch_2_Inv_Sel) ==  CHNonInverted )
                                    Data_Values.Ch_2_Inv_Sel =  CHInverted;
                            else
                                    Data_Values.Ch_2_Inv_Sel =  CHNonInverted;
                            Display_Value = Data_Values.Ch_2_Inv_Sel;
			}
			else if Sel_Key
			{
                            Sel_Fn = Ch_2_Att;
                            Display_Value = Data_Values.Ch_2_Att_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* set up invert variables */
                        CH2_Invert = Data_Values.Ch_2_Inv_Sel;
                        Filter_Parm_Update(Ch_2_Mode);
		}
		/********************************************************************/
		/* Currently in Ch_1_Att Function - act on legal button presses     */
		/********************************************************************/
                else if (Sel_Fn == Ch_1_Att)
		{
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_Att_Sel = Data_Values.Ch_1_Att_Sel - Freq_Speed;
				if ((Data_Values.Ch_1_Att_Sel) <  Att_Min)
                                        Data_Values.Ch_1_Att_Sel =  Att_Min;
				Display_Value = Data_Values.Ch_1_Att_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_Att_Sel = Data_Values.Ch_1_Att_Sel + Freq_Speed;
				if ((Data_Values.Ch_1_Att_Sel) >  Att_Max)
                                        Data_Values.Ch_1_Att_Sel =  Att_Max;
				Display_Value = Data_Values.Ch_1_Att_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_2_FL;
				Display_Value = Data_Values.Ch_2_FL_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Ch_1_Mode);
                }
                
		/********************************************************************/
		/* Currently in Ch_2_Att Function - act on legal button presses     */
		/********************************************************************/
                else if (Sel_Fn == Ch_2_Att)
		{
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_2_Att_Sel = Data_Values.Ch_2_Att_Sel - Freq_Speed;
				if ((Data_Values.Ch_2_Att_Sel) <  Att_Min)
                                        Data_Values.Ch_2_Att_Sel =  Att_Min;
				Display_Value = Data_Values.Ch_2_Att_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_2_Att_Sel = Data_Values.Ch_2_Att_Sel + Freq_Speed;
				if ((Data_Values.Ch_2_Att_Sel) >  Att_Max)
                                        Data_Values.Ch_2_Att_Sel =  Att_Max;
				Display_Value = Data_Values.Ch_2_Att_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = G1_Type;
				Display_Value = Data_Values.G1_Type_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Ch_2_Mode);
                }

                /***********************************************************************/
		/* Currently in G1_Type Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == G1_Type)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.G1_Type_Sel = Data_Values.G1_Type_Sel - 1;
				if ((Data_Values.G1_Type_Sel) ==  TwentyFourdB)  /* then jump this*/
                                        Data_Values.G1_Type_Sel = Data_Values.G1_Type_Sel - 1;
				if ((Data_Values.G1_Type_Sel) <  Filter_Min)
                                        Data_Values.G1_Type_Sel =  G_Filter_Max;
				Display_Value = Data_Values.G1_Type_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.G1_Type_Sel = Data_Values.G1_Type_Sel + 1;
				if ((Data_Values.G1_Type_Sel) ==  TwentyFourdB)  /* then jump this*/
                                        Data_Values.G1_Type_Sel = Data_Values.G1_Type_Sel + 1;
				if ((Data_Values.G1_Type_Sel) > G_Filter_Max)
                                    Data_Values.G1_Type_Sel = Filter_Min;
    				Display_Value = Data_Values.G1_Type_Sel;
			}
                        else if ((Sel_Key) && (Data_Values.G1_Type_Sel == NoFilter))
			{
                            /* update the flter parameters */
                            Sel_Fn = G2_Type;
                            Display_Value = Data_Values.G2_Type_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Sel_Fn = G1_Freq;
                            Display_Value = Data_Values.G1_F_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Gen_Para_Mode);
               }

                /***********************************************************************/
		/* Currently in G1_Freq Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == G1_Freq)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.G1_F_Sel = Data_Values.G1_F_Sel - Freq_Speed;
				if ((Data_Values.G1_F_Sel) <  Freq_Min)
                                        Data_Values.G1_F_Sel =  Freq_Min;
				Display_Value = Data_Values.G1_F_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.G1_F_Sel = Data_Values.G1_F_Sel + Freq_Speed;
				if ((Data_Values.G1_F_Sel) > Freq_Max)
                                    Data_Values.G1_F_Sel = Filter_Max;
    				Display_Value = Data_Values.G1_F_Sel;
			}
			else if ((Sel_Key) && (Data_Values.G1_Type_Sel == Peaking))
			{
                            /* update the flter parameters */
                            Sel_Fn = G1_Q;
                            Display_Value = Data_Values.G1_Q_Sel;
			}
                        else if ((Sel_Key) && ((Data_Values.G1_Type_Sel == SixdB) || (Data_Values.G1_Type_Sel == TwelvedB)))
			{
                            /* update the flter parameters */
                            Sel_Fn = G2_Type;
                            Display_Value = Data_Values.G2_Type_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Gen_Para_Mode);
               }

                /***********************************************************************/
		/* Currently in G1_Q Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == G1_Q)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.G1_Q_Sel = Data_Values.G1_Q_Sel - 1;
				if ((Data_Values.G1_Q_Sel) <  Q_Min)
                                        Data_Values.G1_Q_Sel =  Q_Min;
				Display_Value = Data_Values.G1_Q_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.G1_Q_Sel = Data_Values.G1_Q_Sel + 1;
				if ((Data_Values.G1_Q_Sel) > Q_Max)
                                    Data_Values.G1_Q_Sel = Q_Max;
    				Display_Value = Data_Values.G1_Q_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Sel_Fn = G1_Gain;
                            Display_Value = Data_Values.G1_Gain_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Gen_Para_Mode);
               }
                /***********************************************************************/
		/* Currently in G1_Gain Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == G1_Gain)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.G1_Gain_Sel = Data_Values.G1_Gain_Sel - 1;
				if ((Data_Values.G1_Gain_Sel) <  G_Gain_Min)
                                        Data_Values.G1_Gain_Sel = G_Gain_Min;
				Display_Value = Data_Values.G1_Gain_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.G1_Gain_Sel = Data_Values.G1_Gain_Sel + 1;
				if ((Data_Values.G1_Gain_Sel) > G_Gain_Max)
                                    Data_Values.G1_Gain_Sel = G_Gain_Max;
    				Display_Value = Data_Values.G1_Gain_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Sel_Fn = G2_Type;
                            Display_Value = Data_Values.G2_Type_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Gen_Para_Mode);
               }
                /***********************************************************************/
		/* Currently in G2_Type Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == G2_Type)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.G2_Type_Sel = Data_Values.G2_Type_Sel - 1;
				if ((Data_Values.G2_Type_Sel) ==  TwentyFourdB)  /* then jump this*/
                                        Data_Values.G2_Type_Sel = Data_Values.G2_Type_Sel - 1;
				if ((Data_Values.G2_Type_Sel) <  Filter_Min)
                                        Data_Values.G2_Type_Sel =  G_Filter_Max;
				Display_Value = Data_Values.G2_Type_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.G2_Type_Sel = Data_Values.G2_Type_Sel + 1;
				if ((Data_Values.G2_Type_Sel) ==  TwentyFourdB)  /* then jump this*/
                                        Data_Values.G2_Type_Sel = Data_Values.G2_Type_Sel + 1;
				if ((Data_Values.G2_Type_Sel) > G_Filter_Max)
                                    Data_Values.G2_Type_Sel = Filter_Min;
    				Display_Value = Data_Values.G2_Type_Sel;
			}
                        else if ((Sel_Key) && (Data_Values.G2_Type_Sel == NoFilter))
			{
                            /* update the flter parameters */
                            Sel_Fn = Ch_1_FL;
                            Display_Value = Data_Values.Ch_1_FL_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Sel_Fn = G2_Freq;
                            Display_Value = Data_Values.G2_F_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Gen_Para_Mode);
               }

                /***********************************************************************/
		/* Currently in G2_Freq Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == G2_Freq)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.G2_F_Sel = Data_Values.G2_F_Sel - Freq_Speed;
				if ((Data_Values.G2_F_Sel) <  Freq_Min)
                                        Data_Values.G2_F_Sel =  Freq_Min;
				Display_Value = Data_Values.G2_F_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.G2_F_Sel = Data_Values.G2_F_Sel + Freq_Speed;
				if ((Data_Values.G2_F_Sel) > Freq_Max)
                                    Data_Values.G2_F_Sel = Filter_Max;
    				Display_Value = Data_Values.G2_F_Sel;
			}
			else if ((Sel_Key) && (Data_Values.G2_Type_Sel == Peaking))
			{
                            /* update the flter parameters */
                            Sel_Fn = G2_Q;
                            Display_Value = Data_Values.G2_Q_Sel;
			}
                        else if ((Sel_Key) && ((Data_Values.G2_Type_Sel == SixdB) || (Data_Values.G2_Type_Sel == TwelvedB)))
			{
                            /* update the flter parameters */
                            Sel_Fn = Ch_1_FL;
                            Display_Value = Data_Values.Ch_1_FL_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Sel_Fn = G2_Q;
                            Display_Value = Data_Values.G2_Q_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Gen_Para_Mode);
               }

                /***********************************************************************/
		/* Currently in G2_Q Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == G2_Q)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.G2_Q_Sel = Data_Values.G2_Q_Sel - 1;
				if ((Data_Values.G2_Q_Sel) <  Q_Min)
                                        Data_Values.G2_Q_Sel =  Q_Min;
				Display_Value = Data_Values.G2_Q_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.G2_Q_Sel = Data_Values.G2_Q_Sel + 1;
				if ((Data_Values.G2_Q_Sel) > Q_Max)
                                    Data_Values.G2_Q_Sel = Q_Max;
    				Display_Value = Data_Values.G2_Q_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Sel_Fn = G2_Gain;
                            Display_Value = Data_Values.G2_Gain_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Gen_Para_Mode);
               }
                /***********************************************************************/
		/* Currently in G2_Gain Function - act on legal button presses         */
		/***********************************************************************/
                else if (Sel_Fn == G2_Gain)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.G2_Gain_Sel = Data_Values.G2_Gain_Sel - 1;
				if ((Data_Values.G2_Gain_Sel) <  G_Gain_Min)
                                        Data_Values.G2_Gain_Sel = G_Gain_Min;
				Display_Value = Data_Values.G2_Gain_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.G2_Gain_Sel = Data_Values.G2_Gain_Sel + 1;
				if ((Data_Values.G2_Gain_Sel) > G_Gain_Max)
                                    Data_Values.G2_Gain_Sel = G_Gain_Max;
    				Display_Value = Data_Values.G2_Gain_Sel;
			}
			else if Sel_Key
			{
                            /* update the flter parameters */
                            Sel_Fn = Ch_1_FL;
                            Display_Value = Data_Values.Ch_1_FL_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the flter parameters */
                Filter_Parm_Update(Gen_Para_Mode);
               }

                /********************************************************************/
		/* Currently in Volume_Sel_Fn Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Volume_Sel_Fn)
		{
                    if(Fast_Change != 1)
                        Fast_Change = 3;
			/* Down */
			if Down_Keys
			{
				if (Data_Values.Volume_Gain_Sel >=  (Volume_Min + Fast_Change))
				{
					Data_Values.Volume_Gain_Sel = Data_Values.Volume_Gain_Sel - Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Volume_Gain_Sel;
                                /* update the flter parameters */
                                Filter_Parm_Update(Ch_1_Mode);
                                Filter_Parm_Update(Ch_2_Mode);			
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Volume_Gain_Sel <= (Volume_Max - Fast_Change))
				{
					Data_Values.Volume_Gain_Sel = Data_Values.Volume_Gain_Sel + Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Volume_Gain_Sel;
                                /* update the flter parameters */
                                Filter_Parm_Update(Ch_1_Mode);
                                Filter_Parm_Update(Ch_2_Mode);
                        }
			else if Sel_Key
			{
				Display_Value = Data_Values.Volume_Gain_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		}

                /************************************************************************/
		/* Currently in SAVE Function, act on legal button presses				*/
		/************************************************************************/
		else if(Sel_Fn == Save_Sel_Fn)
		{
			/*		This section miplements the save function 					*/
			/* Up 																*/
			if Up_Keys
			{
				Memory_Bank += 1;
				if (Memory_Bank > Max_Mem_Banks_With_Erase)
					Memory_Bank = Default_Mem_Bank;
				Display_Value = Memory_Bank;
			}
			/* Down */
			else if Down_Keys
			{
				if (Memory_Bank == Default_Mem_Bank)
					Memory_Bank = Max_Mem_Banks;
				else
					Memory_Bank -= 1;
				Display_Value = Memory_Bank;
			}
			/* Select Key hit */
                        /* write memory for normal save operations */
                        else if (Sel_Key && (Memory_Bank <= Max_Mem_Banks))
			{

                        /* set SPI 1 port into normal SPI mode*/

                        //Disable SPI1 interrupt.
//                        mSPI2IntDisable();
//                        INTClearFlag(INT_SPI2);

 //                       SpiChnClose(SPI_CHANNEL1);
//                        DelayMs(Clear_Buffers_SPI_Time);

//                        SpiChnOpen(1,SPICON_MSTEN | SPICON_CKP | SPICON_ON | SPI_OPEN_MODE8, 256);
 //                       DelayMs(Clear_Buffers_SPI_Time);

			/********************************************************************/
			/*		Save the data                                       */
			/********************************************************************/
                        Channel_Mem_Offset =  Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + Volume_Gain_Offset, Data_Values.Volume_Gain_Sel);

                                HDWordWriteSPI(Channel_Mem_Offset + FL_Offset, Data_Values.Ch_1_FL_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + SL_Offset, Data_Values.Ch_1_SL_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + FH_Offset, Data_Values.Ch_1_FH_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + SH_Offset, Data_Values.Ch_1_SH_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Inv_Offset, Data_Values.Ch_1_Inv_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Del_Offset, Data_Values.Ch_1_Del_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Att_Offset, Data_Values.Ch_1_Att_Sel);

                                Channel_Mem_Offset =  Channel_Set_Array_Size + Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + FL_Offset, Data_Values.Ch_2_FL_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + SL_Offset, Data_Values.Ch_2_SL_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + FH_Offset, Data_Values.Ch_2_FH_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + SH_Offset, Data_Values.Ch_2_SH_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Inv_Offset, Data_Values.Ch_2_Inv_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Del_Offset, Data_Values.Ch_2_Del_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Att_Offset, Data_Values.Ch_2_Att_Sel);

                                Channel_Mem_Offset =  Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + G1_F_Offset, Data_Values.G1_F_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + G1_Type_Offset, Data_Values.G1_Type_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + G1_Q_Offset, Data_Values.G1_Q_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + G1_Gain_Offset, Data_Values.G1_Gain_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + G2_F_Offset, Data_Values.G2_F_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + G2_Type_Offset, Data_Values.G2_Type_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + G2_Q_Offset, Data_Values.G2_Q_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + G2_Gain_Offset, Data_Values.G2_Gain_Sel);



//ConfigIntSPI2(SPI_FAULT_INT_DIS | SPI_TX_INT_EN | SPI_RX_INT_DIS | SPI_INT_SUB_PRI_2 | SPI_INT_PRI_3);
//SpiChnClrIntFlags(2);
//INTEnableInterrupts();

/* turn on channel 1*/
/*spi_con1 = SPI_OPEN_MSTEN | //Master mode enable
        SPI_OPEN_SSEN | //Enable slave select function
        SPI_OPEN_CKP_HIGH | //Clock polarity Idle High Actie Low
        SPI_OPEN_ENHBUF | // Enhanced Mode
        SPI_OPEN_TBE_HALF_EMPTY | //generate event when TX buffer more than half empty
        SPI_OPEN_MODE16 | //Data mode: 24b
        SPI_OPEN_MODE32 | //Data mode: 24b
        SPI_OPEN_MCLKSEL | //Clock selected is reference clock
        SPI_OPEN_FSP_HIGH; //Frame Sync Pulse is active high

spi_con2 = SPI_OPEN2_AUDEN | //Enable Audio mode
        SPI_OPEN2_AUDMOD_I2S; //Enable I2S mode

SpiChnOpenEx(SPI_CHANNEL1, spi_con1, spi_con2, (mclk / baud_rate));

SPI2CON2 = SPI2CON2 & 0xfffffdff; // ignore receive overflow
SPI2STATCLR = 1<<6; // clear receive overflow
SPI2CON = SPI2CON | 0x00008000; // turn SPI2 back on
SPI2BUF = 0x00000000;
SPI2BUF = 0x00000000;



SpiChnOpenEx(SPI_CHANNEL1, spi_con1, spi_con2, (mclk / baud_rate));
*/


				Sel_Fn = Choosing_Fn;
				Mode = Ch_1_Mode;
				Display_Value = Mode;
			}
                        /* Exit */
			else if (Sel_Key && (Memory_Bank == Mem_Banks_Do_Not_Erase))
			{
                                Memory_Bank = Default_Mem_Bank;
                                Step_Size = 0;
				Sel_Fn = Choosing_Fn;
				Mode = Ch_1_Mode;
				Display_Value = Mode;
			}
                        else if (Sel_Key && (Memory_Bank == Mem_Banks_Erase))
			{
                         /* set SPI 1 port into normal SPI mode*/
                            SpiChnClose(SPI_CHANNEL1);
                            SpiChnOpen(1,SPICON_MSTEN | SPICON_CKP | SPICON_ON, 256);
                            DelayMs(Clear_Buffers_SPI_Time);
                            for (Memory_Bank = Default_Mem_Bank; Memory_Bank <=  Max_Mem_Banks; Memory_Bank++)
                            {
                                /********************************************************************/
                                /* Set about resetting memory to defaults                           */
                                /* Refer to header file for default values...                       */
				/********************************************************************/
                                Channel_Mem_Offset =  Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + FL_Offset, CH_1_FL_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + SL_Offset, CH_1_SL_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + FH_Offset, CH_1_FH_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + SH_Offset, CH_1_SH_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Inv_Offset, CH_1_Inv_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Del_Offset, CH_1_Del_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Att_Offset, CH_1_Att_Default);

                                Channel_Mem_Offset =  Channel_Set_Array_Size + Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + FL_Offset, CH_2_FL_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + SL_Offset, CH_2_SL_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Inv_Offset, CH_2_Inv_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Del_Offset, CH_2_Del_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Att_Offset, CH_2_Att_Default);

                                Channel_Mem_Offset =  Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + G1_F_Offset, G1_Freq_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + G1_Type_Offset, G1_Type_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + G1_Q_Offset, G1_Q_Default );
                                HDWordWriteSPI(Channel_Mem_Offset + G1_Gain_Offset, G1_Gain_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + G2_F_Offset, G2_Freq_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + G2_Type_Offset, G2_Type_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + G2_Q_Offset, G2_Q_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + G2_Gain_Offset, G2_Gain_Default);

                                HDWordWriteSPI(Channel_Mem_Offset + Volume_Gain_Offset, Volume_Gain_Default);
                            }
                            //Configure SPI in I2S mode with 24-bit stereo audio.
                            spi_con1 = SPI_OPEN_MSTEN | //Master mode enable
                                    SPI_OPEN_SSEN | //Enable slave select function
                                    SPI_OPEN_CKP_HIGH | //Clock polarity Idle High Actie Low
                                    SPI_OPEN_ENHBUF | // Enhanced Mode
                                    SPI_OPEN_TBE_HALF_EMPTY | //generate event when TX buffer more than half empty
                                    SPI_OPEN_MODE16 | //Data mode: 24b
                                    SPI_OPEN_MODE32 | //Data mode: 24b
                                    SPI_OPEN_MCLKSEL | //Clock selected is reference clock
                                    SPI_OPEN_FSP_IN; //Frame Sync Pulse is active high

                            spi_con2 = SPI_OPEN2_AUDEN | //Enable Audio mode
                                    SPI_OPEN2_AUDMOD_I2S; //Enable I2S mode
                            SpiChnOpenEx(SPI_CHANNEL1, spi_con1, spi_con2, (mclk / baud_rate));

                            Memory_Bank = Default_Mem_Bank;
                            Sel_Fn = Choosing_Fn;
                            Mode = Ch_1_Mode;
                            Display_Value = Mode;
			}

			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = Choosing_Fn;
				Mode = Ch_1_Mode;
				Display_Value = Mode;
			}
			else;
		}

		/************************************************************************/
		/* Currently in LOAD Function, act on legal button presses 				*/
		/************************************************************************/
		else if(Sel_Fn == Load_Sel_Fn)
		{
			/********************************************************************/
			/*		This section miplements the load data function 				*/
			/********************************************************************/
			/* Up */
			if Up_Keys
			{
				Memory_Bank += 1;
				if (Memory_Bank > Max_Mem_Banks)
					Memory_Bank = 0;
				Display_Value = Memory_Bank;
			}
			/* Down */
			else if Down_Keys
			{
				if (Memory_Bank == 0)
					Memory_Bank = Max_Mem_Banks;
				else
					Memory_Bank -= 1;
				Display_Value = Memory_Bank;
			}
			/* Select Key hit */
			else if Sel_Key
			{

                                /* set SPI 1 port into normal SPI mode*/
                                SpiChnClose(SPI_CHANNEL1);
                                SpiChnOpen(1,SPICON_MSTEN | SPICON_CKP | SPICON_ON, 256);
                                DelayMs(Clear_Buffers_SPI_Time);
                                /***********************************************/
				/*		Load the data                   */
				/***********************************************/		
				Load_From_Memory(Memory_Bank); /* Read Data from ROM */
				/* now program the values the user loaded...*/
                                Filter_Parm_Update(Gen_Para_Mode);
                                Filter_Parm_Update(Ch_1_Mode);
                                Filter_Parm_Update(Ch_2_Mode);

				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;

                                //Configure SPI in I2S mode with 24-bit stereo audio.
                                spi_con1 = SPI_OPEN_MSTEN | //Master mode enable
                                    SPI_OPEN_SSEN | //Enable slave select function
                                    SPI_OPEN_CKP_HIGH | //Clock polarity Idle High Actie Low
                                    SPI_OPEN_ENHBUF | // Enhanced Mode
                                    SPI_OPEN_TBE_HALF_EMPTY | //generate event when TX buffer more than half empty
                                    SPI_OPEN_MODE16 | //Data mode: 24b
                                    SPI_OPEN_MODE32 | //Data mode: 24b
                                    SPI_OPEN_MCLKSEL | //Clock selected is reference clock
                                    SPI_OPEN_FSP_IN; //Frame Sync Pulse is active high

                                spi_con2 = SPI_OPEN2_AUDEN | //Enable Audio mode
                                    SPI_OPEN2_AUDMOD_I2S; //Enable I2S mode
                                SpiChnOpenEx(SPI_CHANNEL1, spi_con1, spi_con2, (mclk / baud_rate));
                        }

                        /************************************************************************/
                	/* Exit                                                                 */
                	/************************************************************************/
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
                }


		/************************************************************************/
		/* Else key not pressed, and nothing to display                         */
		/************************************************************************/
		else
                    Update_Display = 0;

	
		if (Update_Display)	
			Display_Data(Sel_Fn, Display_Value);
		else;


	}  /* Done changing values for the user */


	/* Clears the flag saying that the push button press has been addressed */
	Step_Size = 0;

	Auto_Revert = Auto_Revert - 1;   /* If no key pressed, then decrement this counter */
	if 	(Auto_Revert <= 0)   	/* Then force the system to volume mode */
		{
                        /* If key pressed, reset the counter */
                        Auto_Revert = Auto_Revert_Init_Val;
			Band = 0;
                        Sel_Fn = Choosing_Fn;
			Display_Value = Mode;
			Display_Data(Sel_Fn, Display_Value);
		}

        /* delay in main loop to allow the HMI to work at a sane speed */
        DelayUs(Key_Press_Delay_Us);

	}
        /* and just stay in this loop till forever...*/
        while (1);
}
