/* 28/6/07                                                  */
/* End of build 1 of major rewrite to implement rotary ctrl */
/* basic functionality for:
	- Load and save three parm sets
	- Rotary control basics
	- Memory map for load save
	- menu structure for rotary control
*/
/* 28/10/2013                                               */
/* Added a bunch of stuff for PIC32MX450F256H               */
/* includes significant update to use generic defines       */
/* and reduces processor dependance of code used            */
/* 24/12/2013                                               */
/* Generally all working - bar tweaks                       */
/*                                                          */

// Let compile time pre-processor calculate the PR1 (period)
#define SYS_FREQ             (96000000L)
#define PB_DIV                 1
#define PRESCALE               256
#define TOGGLES_PER_SEC        1
#define T1_TICK               (SYS_FREQ/PB_DIV/PRESCALE/TOGGLES_PER_SEC)

//Apears this version of the compiler omits this definition
// if you get compiler errors, then these have probably turned back up :)
#define IN_FN_PPS2_RPD3 RPD3Rbits.RPD3R
#define IN_PIN_PPS2_SDI1 0
#define IN_FN_PPS2_RPG7 RPG7Rbits.RPG7R
#define IN_PIN_PPS2_SDI2 6

/* RB0 - spare (output)*/
/* RB1 - spare (output)*/
/* RB2 - spare (output)*/
/* RB3 - spare (output)*/
/* RB4 - spare (output)*/
/* RB5 - spare (output)*/
/* RB6 - Rotary_Encoder_Select_0 (input)*/
#define Rotary_Encoder_Select_0 64
/* RB7 - Rotary_Encoder_Select_1 (input)*/
#define Rotary_Encoder_Select_1 128

/* Old Rotary Encoder = 00XXXXXX*/
#define Old_Rotary_Encoder 0x0000
/* Type A rotary Encoder = 10XXXXXX*/
#define Type_A_Rotary_Encoder 0x0080
/* Type B Rotary Encoder = 01XXXXXX*/
#define Type_B_Rotary_Encoder 0x0040

/* RB8 - LCD0 (output) */
/* RB9 - LCD1 (output) */
/* RB10 - LCD2 (output) */
/* RB11 - LCD3 (output) */
/* RB12 - LCD4 (output) */
/* RB13 - LCD5 (output) */
/* RB14 - LCD6 (output) */
/* RB15 - LCD7 (output) */
#define Clear_LCD 1
#define Home_LCD 2
#define No_Blink_LCD 8
#define Blink_LCD 14

/* Function Select 							*/
/* 	RS	RW 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	1	1	1	0	0	0	*/
#define LCD_Function 0x38

/* Display On */
/* 	RS	RW 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	1	1	1	1	*/
#define LCD_Display_On 0x0F

/* Display Clear */
/* 	RS	RW 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	0	0	0	1	*/
#define LCD_Display_Clear 0x01

/* Entry Mode */
/* 	RS	RW 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	0	1	1	1	*/
#define LCD_Display_Entry_Mode 0x06

/* DDRAM Line 1 Address */
/* 	RS(d6)	RW(d5) 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	1	0	0	0	0	0	0	0	*/

#define LCD_Line_1 0x80

/* DDRAM Line 2 Address */
/* 	RS(d6)	RW(d5) 	E(d7)	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	1	1	0	1       0	0	0	0	0	*/

#define LCD_Line_2      0xC0

/* Enable Write Data to LCD */
/* 	RS(d6)	RW(d5) 	E(d7)	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	1	0	0	d7	d6	d5	d4	d3	d2	d1	d0	*/
#define Write_Data_Enable_LCD 64

/* Write Data to LCD */
/* 	RS(d6)	RW(d5) 	E(d7)	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	1	0	1	d7	d6	d5	d4	d3	d2	d1	d0	*/
#define Write_Data_LCD 192
#define Write_Control_LCD 32
#define Write_Control_Enable_LCD 128
#define	LCD_Data(x)	{ int _dt; \
                          DelayUs(1);\
			  _dt = x<<8; \
                          _dt = _dt + (PORTB & 0x00FF);\
                          PORTB = _dt;\
                          DelayUs(1);}

#define Write_Control_LCD_CSEL DelayUs(1); PORTD = PORTD & (0xFFFF - (Write_Data_Enable_LCD + Write_Control_Enable_LCD + Write_Control_LCD)); DelayUs(1)
#define Write_Control_Enable_LCD_CSEL DelayUs(1); PORTD = PORTD | Write_Control_Enable_LCD; DelayUs(1)
#define Write_Data_LCD_CSEL DelayUs(1); PORTD = PORTD | (Write_Control_Enable_LCD + Write_Data_Enable_LCD); DelayUs(1); PORTD = PORTD & (0xFFFF - Write_Control_LCD);DelayUs(50)
#define Write_Data_Enable_LCD_CSEL DelayUs(1); PORTD = PORTD | Write_Data_Enable_LCD; DelayUs(1); PORTD = PORTD & (0xFFFF - Write_Control_Enable_LCD - Write_Control_LCD); DelayUs(50)
// LCD wants 37uS delay after write
#define LCD_Delay_After_Write 40 //40
// LCD Data Setting Time - use 5uS
#define LCD_Settle_Before_Write 5  //5
// LCD Control 1.5mS Setting Time
#define LCD_Settle_After_Control 4 //4


/* port C is oscillators */

/* RD0 SPARE */
/* RD1 = SPI_CS1*/
#define CSel 0x002
/* RD2 = SPI_SCK1*/
/* RD3 = SPI_SDI1 = "0000" = input*/
#define SDI_Out 16
/* RD4 = SDI_SDO1 = "1000"*/
/* RD5 = LCD_RW (output)*/
#define R_W_LCD 32
/* RD6 = LCD_RS (output)*/
#define RS_LCD 64
/* RD7 = LCD_CS (output)*/
#define Enable_LCD 128

/* RD8 = DSP_RESET*/
#define Reset_DSP       0x0100   /* Bit 8 of port D */
/* RD9 = SPI_SS1 - "0111"*/
/* RD10 = RESET_DAC*/
#define Reset_DAC 	0x0400   /* bit 10 of port D */
/* RD11 = UNUSED (output)*/
/*
 * Drive LCD Control for dispaly select using port B Output
 */

/* RE0 = SPARE*/
/* RE1 = MODE_SEL (input)*/
/* RE2 = FN_SEL (input)*/
/* RE3 = PHASEA (input)*/
/* RE4 = PHASEB (input)*/
/* RE5 = SPARE*/
/* RE6 = SPARE*/
/* RE7 = Select_PGA2320*/
#define PGA2320_Select 0x0080

#define Port_E_Button_Pressed   (2+4+8+16)

/* RF0 EEPROM Chip Select_Active Low*/
#define EEPROM_Select 1
/* RF1 EEPROM Chip Hold_Active Low*/
#define EEPROM_Hold 2
/* RF1 SPARE (output)*/
/* RF4 UNUSED (output)*/
/* RF5 UNUSED (output)*/

/* RG6 = SPI_SCK2 */
/* RG7 = SPI_SDI2 "0001" */
/* RG8 = SPI_SDO2 = "0110" */
/* RG9 = SPI_SS2 = "0110" */

/*
PORT C definitions  - NEEDS FIXING
*/
#define Port_XXXX_Button_Pressed   3


/* Input port bits */
/* Input Port E */
#define Band_Select         1
#define Function_Select     2
#define Max_Bands           3


/* Display Strings */
#define Blank_Char_Init 	0x20
#define Line1_Init      	"  TGM Was Here  "
#define Line2_Init     	 	"DSP1.41    Apr14"
#define Line2_Select_Init	"Press Select:   "
#define Line1_Channel_1_Init   	"Channel 1 Setup:"
#define Line1_Channel_2_Init   	"Channel 2 Setup:"
#define Line1_Save_Init 	"Save to bank:   "
#define Line1_Erase_Check 	"Erase Memory?   "
#define Line2_Erase_Check_Y 	"Restore Defaults"
#define Line2_Erase_Check_N 	"NO! Let me out! "
#define Line1_Load_Init 	"Load from bank: "
#define Line1_Level_Init  	"Attenuation:  dB"
#define Line2_Level_Init  	"Ref Max Output  "
#define Boot_Msg 666
#define Line1_Choose_Str_Init	"Select Function:"

/*
Stuff to allow modes to be tracked
Use these to control the state machine
*/
#define Choosing_Fn	0
#define Ch_1_Mode       1
#define Ch_2_Mode 	2
#define Gen_Para_Mode   3
#define Volume_Mode     4
#define Save_Mode	5
#define Load_Mode	6

/*
Stuff to allow functions to be tracked in Channel 1 mode
*/
#define Ch_1_FL         10
#define Ch_1_SL         11
#define Ch_1_FH         12
#define Ch_1_SH         13
#define Ch_1_Del        14
#define Ch_1_Inv        15
#define Ch_1_Att        16


/*
Stuff to allow functions to be tracked in Channel 2 mode
*/
#define Ch_2_FL         20
#define Ch_2_SL         21
#define Ch_2_FH         22
#define Ch_2_SH         23
#define Ch_2_Del        24
#define Ch_2_Inv        25
#define Ch_2_Att        26

/*
Stuff to allow functions to be tracked in Gen filter mode
*/
#define G1_Type         30
#define G1_Freq         31
#define G1_Q            32
#define G1_Gain         33
#define G2_Type         34
#define G2_Freq         35
#define G2_Q            36
#define G2_Gain         37

#define Gen_Freq_Display_Line 1
#define Gen_Q_Display_Line 2
#define Gen_Gain_Display_Line 3
/*
Stuff to allow functions to be tracked in Volume mode
*/
#define Volume_Sel_Fn 	40
#define Volume_Sel_Fn_Choice_Display_Row  3
#define Volume_Sel_Fn_Units_Display_Row  6
/*
Stuff to allow functions to be tracked in Save mode
*/
#define Save_Sel_Fn 	50

/*
Stuff to allow functions to be tracked in Load mode
*/
#define Load_Sel_Fn 	60

/*
 Frequency Speed Function
*/
#define Freq_Speed_Counter_1_fn 	666
#define Freq_Speed_Counter_2_fn 	667


/*
Stuff to allow memory bank in use to be tracked
*/
#define Mem_Banks_Do_Not_Erase 3
#define Mem_Banks_Erase 4
#define Max_Mem_Banks_With_Erase 4
#define Max_Mem_Banks 	2
#define Default_Mem_Bank 0
#define Clear_Buffers_SPI_Time 3

/*
more stuff 
Use this bitfield to watch what buttons are happening...
*/
#define  HMI_Port PORTE
//#define  Vol_Dn (unsigned char) 1
//#define  Vol_Up (unsigned char) 2
//#define  Vals_2 (unsigned char) 32
//#define  Mode_Sel (unsigned char) 64
#define  Exit (unsigned char) 2     //pretty sure this is bit 1 of port E
#define  Sel (unsigned char) 4      //pretty sure this is bit 2 of port E
#define  Vals_0 (unsigned char) 8   //pretty sure this is bit 3 of port E
#define  Vals_1 (unsigned char) 16  //pretty sure this is bit 4 of port E
#define Key_Press_Delay_Us 150
#define Slow_Counter_Init 700
#define Fast_Counter_Init 40

#define Phase_0 (unsigned char) 0
#define Phase_1 Vals_0
#define Phase_2 Vals_0 + Vals_1
#define Phase_3 Vals_1

#define Rot_Up Vals_1
#define Rot_Down Vals_0 + Vals_1

#define Up_Keys (Keypress_Read == Vals_1)
#define Down_Keys (Keypress_Read == (Vals_0 + Vals_1))
#define Sel_Key (Keypress_Read == Sel)
#define Exit_Key (Keypress_Read == Exit)

/*Define Value for Mute Volume */
#define  Mute 255

/* Auto Repeat Counts - how long to wait before starting to Auto Repeat*/
#define Auto_Repeat_Count 6


typedef struct{
		int Ch_1_FL_Sel;     /* Freq is in Hz*/
		int Ch_1_SL_Sel;     /* Lower Filter slope (none, 6, 12 and 24)*/
		int Ch_1_FH_Sel;     /* Freq is in Hz*/
		int Ch_1_SH_Sel;     /* Lower Filter slope (none, 6, 12 and 24)*/
		int Ch_1_Inv_Sel;    /* Inverted*/
		int Ch_1_Del_Sel;    /* Delay in mm*/
		int Ch_1_Att_Sel;    /* Attenuation in tenths of dB*/

		int Ch_2_FL_Sel;     /* Freq is in Hz*/
		int Ch_2_SL_Sel;     /* Lower Filter slope (none, 6, 12 and 24)*/
		int Ch_2_FH_Sel;     /* Freq is in Hz*/
		int Ch_2_SH_Sel;     /* Lower Filter slope (none, 6, 12 and 24)*/
		int Ch_2_Inv_Sel;    /* Inverted*/
		int Ch_2_Del_Sel;    /* Delay in mm*/
		int Ch_2_Att_Sel;    /* Attenuation in tenths of dB*/

		int G1_Type_Sel;     /* G1 types are: none, LP6, LP12, Para*/
                int G1_F_Sel;        /* Freq is in Hz*/
		int G1_Q_Sel;        /* Q is in setps of 0.1*/
		int G1_Gain_Sel;     /* Gain is in tenths of dB*/
		int G2_Type_Sel;     /* G1 types are: none, LP6, LP12, Para*/
		int G2_F_Sel;        /* Freq is in Hz*/
		int G2_Q_Sel;        /* Q is in setps of 0.1*/
		int G2_Gain_Sel;     /* Gain is in tenths of dB*/

                int Volume_Gain_Sel; /* volume attenuation in dB */

}  data_val_struct ;

#define Default_Mem_Bank 0

/* How big is each set of data in ROM?*/
#define ParmSet_Array_Size          256
#define Channel_Set_Array_Size      64

/* crossover and para data locations */
#define FL_Offset              0
#define SL_Offset              4
#define FH_Offset              8
#define SH_Offset              12
#define Inv_Offset             16
#define Del_Offset             20
#define Att_Offset             24
#define G1_F_Offset            128
#define G1_Type_Offset         132
#define G1_Q_Offset            136
#define G1_Gain_Offset         140
#define G2_F_Offset            160
#define G2_Type_Offset         164
#define G2_Q_Offset            168
#define G2_Gain_Offset         172

#define Volume_Gain_Offset     200



/* Sample rate of the DAC and thus the DDS timing cycle...*/
#define Sample_Rate 46875  //this is 3000000/64
//#define Sample_Rate 62500  //this is 4000000/64
#define baud_rate (Sample_Rate * 64)
#define mclk (Sample_Rate * 256)
//Definitions - relate to Clock Rate and Sample Rate.
// TEST #define RODIV_PROG	4 - this gives you fs of about 46KHz
#define RODIV_PROG	4
#define REFTRIM		0

#define Number_Clicks_Freq_Speed_Up 5
#define Max_Freq_Speed 10000
#define Auto_Revert_Init_Val 400000

#define EEPROM_WR_Delay 10  /* Approx EEPROM write delay in mS */ 

/* Addresses of things in the system... */
#define Write_SPI 0X00

/* Can't believe I have to define this!!! */
#define Pi (double) 3.141592653590
#define sqrt2 (double) 1.414213562373

/* Savy typing and make things clearer in the code */
#define ClearDSPReset PORTD = PORTD | Reset_DSP
#define SetDSPReset PORTD = PORTD & (0xFFFF - Reset_DSP)
#define ClearDACReset PORTD = PORTD | Reset_DAC
#define SetDACReset PORTD = PORTD & (0xFFFF - Reset_DAC)
#define DAC_Reset_Time 20
#define More_Than_1_Audio_Frame_uS 32
#define CselLow DelayUs(1); PORTD = PORTD & (0xFFFF - CSel); DelayUs(1)
#define CselHigh DelayUs(1); PORTD = PORTD | (CSel); DelayUs(1)
#define CselLowPGA2320 DelayUs(1); PORTE = PORTE & (0xFFFF - PGA2320_Select); DelayUs(1)
#define CselHighPGA2320 DelayUs(1); PORTE = PORTE | (PGA2320_Select); DelayUs(1)
#define CselEEPROM DelayUs(1); PORTF = PORTF | (EEPROM_Select); DelayUs(1)
#define CselClearEEPROM DelayUs(1); PORTF = PORTF & (0xFFFF - EEPROM_Select); DelayUs(1)
#define HoldClearEEPROM DelayUs(1); PORTF = PORTF | (EEPROM_Hold); DelayUs(1)
#define HoldEEPROM DelayUs(1); PORTF = PORTF & (0xFFFF - EEPROM_Hold); DelayUs(1)

//#define Write_8_Bits(Data_Word) WriteSPI(Data_Word,8)
//#define Write_16_Bits(Data_Word) WriteSPI(Data_Word,16)
//#define Write_32_Bits(Data_Word) WriteSPI(Data_Word,32)
#define Write_SPI_Addr(Device_Address, Write_Bit) Write_8_Bits(((Device_Address << 1)+Write_Bit))
#define Write_To_SPI 0
#define Read_From_SPI 1
#define Fast_Change_Frequencies 500
#define Fast_Change_Delays 100

/* Old Rotary Encoder = 00XXXXXX*/
#define Encoder_Type_Is_Old (!(PORTB & 0x00C0))
/* Type A rotary Encoder = 10XXXXXX*/
#define Encoder_Type_Is_TypeA (PORTB & Type_A_Rotary_Encoder)
/* Type B Rotary Encoder = 01XXXXXX*/
#define Encoder_Type_Is_TypeB (PORTB & Type_B_Rotary_Encoder)

#define SpeedOfSound 341210   /* millimetres per second */


/* defeine Filter Parameters*/
#define CHInverted  -1
#define CHNonInverted  1
#define NoFilter 109
#define Filter_Min NoFilter
#define SixdB    110
#define TwelvedB 111
#define TwentyFourdB 112
#define Filter_Max  TwentyFourdB
#define Peaking  113
#define G_Filter_Max Peaking

#define LowPass  100
#define HighPass 101
/* note IIR Scaling needs to leave enough "integer bits" in the 32 bit word*/
/* to accomodate te larger of filter gain (10 time for 20dB gain) and 1/Q*/
#define IIR_Scaling  67108864  /*2^26*/
#define DAC_Output_Max 8388607 /* 2^23 - 1*/
#define DAC_Valid_BitMask 0xFF800000
#define DAC_Overflow_Bitmask 0x7F000000
/* hardcoded Filter Types for F2, F3, F4, F5, F6 and F7*/
#define F0_LP_Type LowPass
#define F1_LP_Type LowPass
#define F0_Peaking Peaking
#define F1_Peaking Peaking
#define F2_Type HighPass
#define F3_Type HighPass
#define F4_Type LowPass
#define F5_Type LowPass
#define F6_Type HighPass
#define F7_Type HighPass
#define Q_BW   0.707


/*  Limit Values for Data Structures  */
#define	CH_1_FL_Default     10
#define	Freq_Min            10
#define	Freq_Max            20000
#define	CH_1_SL_Default     TwelvedB
#define	CH_1_FH_Default     100
#define	CH_1_SH_Default     TwelvedB
#define	CH_1_Inv_Default    CHNonInverted
#define	CH_1_Del_Default    1
#define	Del_Min             1
#define	Del_Max             2040 /*samples - this is a bit less than 2048 samples...*/
#define	CH_1_Att_Default    0
#define	Att_Min             -100   /* this is in tenths of dB*/
#define	Att_Max             +100   /* this is in tenths of dB*/
#define	Q_Min               1   /* this is tenths of units*/
#define	Q_Max               +100   /* this is tenths of Units*/
#define	G_Gain_Min          -100   /* this is tenths of dB*/
#define	G_Gain_Max          +100   /* this is tenths of dB*/
#define	CH_2_FL_Default     100
#define	CH_2_SL_Default     TwelvedB
#define	CH_2_Inv_Default    CHNonInverted
#define	CH_2_Del_Default    1
#define	CH_2_Att_Default    0
#define	Volume_Min          -198   /* this is in halves of of dB*/
#define	Volume_Max          0   /* this is halves of dB*/

#define	G1_Freq_Default     1000
#define	G1_Type_Default     NoFilter
#define	G1_Q_Default        1
#define	G1_Gain_Default     0
#define	G2_Freq_Default     1000
#define	G2_Type_Default     NoFilter
#define	G2_Q_Default        1
#define	G2_Gain_Default     0

#define	Volume_Gain_Default      -10

#define Delay_Buf_Size 2048
#define Delay_Buf_Size_Mask 0x000007ff

#define Min_Freq_Steps  1
#define Max_Freq_Steps  100

#define Acc_Reg_Shift  Acc_Reg = (Acc_Reg >>26);

#define Yn_L_Assignment Yn_L = (int)Acc_Reg
#define Yn_R_Assignment Yn_R = (int)Acc_Reg
#define Y1n_L_Assignment Y1n_L = (int)Acc_Reg
#define Y1n_R_Assignment Y1n_R = (int)Acc_Reg
#define Y2n_L_Assignment Y2n_L = (int)Acc_Reg
#define Y2n_R_Assignment Y2n_R = (int)Acc_Reg
#define Y3n_L_Assignment Y3n_L = (int)Acc_Reg
#define Y3n_R_Assignment Y3n_R = (int)Acc_Reg
#define Y4n_L_Assignment Y4n_L = (int)Acc_Reg
#define Y4n_R_Assignment Y4n_R = (int)Acc_Reg
#define Y5n_L_Assignment Y5n_L = (int)Acc_Reg
#define Y5n_R_Assignment Y5n_R = (int)Acc_Reg
#define Y6n_L_Assignment Y6n_L = (int)Acc_Reg
#define Y6n_R_Assignment Y6n_R = (int)Acc_Reg
#define Y7n_L_Assignment Y7n_L = (int)Acc_Reg
#define Y7n_R_Assignment Y7n_R = (int)Acc_Reg


void	Display_Char(char, char);

void 	Print_Data(int, unsigned char*, unsigned int);

void Load_From_Memory(char );